@@ -5,7 +5,7 @@ use warnings;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(
-	offset offset2 
+	offset 
 	offset_ex offset2_ex
     diff_ex diff union_ex intersection_ex 
     JT_ROUND JT_MITER JT_SQUARE 
