@@ -47,7 +47,8 @@ using TPoint = typename PointType<remove_cvref_t<Shape>>::Type;
 
 /// Getting the coordinate data type for a geometry class.
 template<class GeomClass> struct CoordType { 
-    using Type = typename CoordType<TPoint<GeomClass>>::Type; 
+    using Type = typename CoordType<TPoint<GeomClass>>::Type;
+    static const constexpr Type MM_IN_COORDS = Type{1};
 };
 
 /// TCoord<GeomType> as shorthand for typename `CoordType<GeomType>::Type`.
@@ -127,22 +128,32 @@ template<class S> struct ContourType<DefaultMultiShape<S>> {
     using Type = typename ContourType<S>::Type;
 };
 
-enum class Orientation {
-    CLOCKWISE,
-    COUNTER_CLOCKWISE
-};
+enum class Orientation { CLOCKWISE, COUNTER_CLOCKWISE };
 
 template<class S>
 struct OrientationType {
 
     // Default Polygon orientation that the library expects
-    static const Orientation Value = Orientation::CLOCKWISE;
+    static const constexpr Orientation Value = Orientation::CLOCKWISE;
 };
 
-template<class T> inline /*constexpr*/ bool is_clockwise() { 
+template<class T> inline constexpr bool is_clockwise() {
     return OrientationType<TContour<T>>::Value == Orientation::CLOCKWISE; 
 }
 
+template<class T>
+inline const constexpr Orientation OrientationTypeV =
+    OrientationType<TContour<T>>::Value;
+
+enum class Closure { OPEN, CLOSED };
+
+template<class S> struct ClosureType {
+    static const constexpr Closure Value = Closure::CLOSED;
+};
+
+template<class T>
+inline const constexpr Closure ClosureTypeV =
+    ClosureType<TContour<T>>::Value;
 
 /**
  * \brief A point pair base class for other point pairs (segment, box, ...).
@@ -473,8 +484,8 @@ inline _Box<P> _Box<P>::infinite(const P& center) {
     
     // It is important for Mx and My to be strictly less than half of the
     // range of type C. width(), height() and area() will not overflow this way.
-    C Mx = C((std::numeric_limits<C>::lowest() + 2 * getX(center)) / 2.01);
-    C My = C((std::numeric_limits<C>::lowest() + 2 * getY(center)) / 2.01);
+    C Mx = C((std::numeric_limits<C>::lowest() + 2 * getX(center)) / 4.01);
+    C My = C((std::numeric_limits<C>::lowest() + 2 * getY(center)) / 4.01);
     
     ret.maxCorner() = center - P{Mx, My};
     ret.minCorner() = center + P{Mx, My};
@@ -490,8 +501,9 @@ inline P _Box<P>::center() const BP2D_NOEXCEPT {
     using Coord = TCoord<P>;
 
     P ret = { // No rounding here, we dont know if these are int coords
-        Coord( (getX(minc) + getX(maxc)) / Coord(2) ),
-        Coord( (getY(minc) + getY(maxc)) / Coord(2) )
+        // Doing the division like this increases the max range of x and y coord
+        getX(minc) / Coord(2) + getX(maxc) / Coord(2),
+        getY(minc) / Coord(2) + getY(maxc) / Coord(2)
     };
 
     return ret;
@@ -586,9 +598,9 @@ inline void reserve(RawPath& p, size_t vertex_capacity, const PathTag&)
 }
 
 template<class S, class...Args>
-inline void addVertex(S& sh, const PathTag&, Args...args)
+inline void addVertex(S& sh, const PathTag&, const TPoint<S> &p)
 {
-    sh.emplace_back(std::forward<Args>(args)...);
+    sh.emplace_back(p);
 }
 
 template<class S, class Fn>
@@ -840,9 +852,9 @@ template<class P> auto rbegin(P& p) -> decltype(_backward(end(p)))
     return _backward(end(p));
 }
 
-template<class P> auto rcbegin(const P& p) -> decltype(_backward(end(p)))
+template<class P> auto rcbegin(const P& p) -> decltype(_backward(cend(p)))
 {
-    return _backward(end(p));
+    return _backward(cend(p));
 }
 
 template<class P> auto rend(P& p) -> decltype(_backward(begin(p)))
@@ -872,16 +884,16 @@ inline void reserve(T& sh, size_t vertex_capacity) {
     reserve(sh, vertex_capacity, Tag<T>());
 }
 
-template<class S, class...Args>
-inline void addVertex(S& sh, const PolygonTag&, Args...args)
+template<class S>
+inline void addVertex(S& sh, const PolygonTag&, const TPoint<S> &p)
 {
-    addVertex(contour(sh), PathTag(), std::forward<Args>(args)...);
+    addVertex(contour(sh), PathTag(), p);
 }
 
-template<class S, class...Args> // Tag dispatcher
-inline void addVertex(S& sh, Args...args)
+template<class S> // Tag dispatcher
+inline void addVertex(S& sh, const TPoint<S> &p)
 {
-    addVertex(sh, Tag<S>(), std::forward<Args>(args)...);
+    addVertex(sh, Tag<S>(), p);
 }
 
 template<class S>
