@@ -1,9 +1,8 @@
 #include "Arrange.hpp"
-#include "SVG.hpp"
 
 #include "BoundingBox.hpp"
 
-#include <libnest2d/backends/clipper/geometries.hpp>
+#include <libnest2d/backends/libslic3r/geometries.hpp>
 #include <libnest2d/optimizers/nlopt/subplex.hpp>
 #include <libnest2d/placers/nfpplacer.hpp>
 #include <libnest2d/selections/firstfit.hpp>
@@ -54,23 +53,22 @@ namespace Slic3r {
 
 template<class Tout = double, class = FloatingOnly<Tout>, int...EigenArgs>
 inline constexpr Eigen::Matrix<Tout, 2, EigenArgs...> unscaled(
-    const ClipperLib::IntPoint &v) noexcept
+    const Slic3r::ClipperLib::IntPoint &v) noexcept
 {
-    return Eigen::Matrix<Tout, 2, EigenArgs...>{unscaled<Tout>(v.X),
-                                                unscaled<Tout>(v.Y)};
+    return Eigen::Matrix<Tout, 2, EigenArgs...>{unscaled<Tout>(v.x()),
+                                                unscaled<Tout>(v.y())};
 }
 
 namespace arrangement {
 
 using namespace libnest2d;
-namespace clppr = ClipperLib;
 
 // Get the libnest2d types for clipper backend
-using Item         = _Item<clppr::Polygon>;
-using Box          = _Box<clppr::IntPoint>;
-using Circle       = _Circle<clppr::IntPoint>;
-using Segment      = _Segment<clppr::IntPoint>;
-using MultiPolygon = TMultiShape<clppr::Polygon>;
+using Item         = _Item<ExPolygon>;
+using Box          = _Box<Point>;
+using Circle       = _Circle<Point>;
+using Segment      = _Segment<Point>;
+using MultiPolygon = ExPolygons;
 
 // Summon the spatial indexing facilities from boost
 namespace bgi = boost::geometry::index;
@@ -127,8 +125,8 @@ template<class TBin>
 class AutoArranger {
 public:
     // Useful type shortcuts...
-    using Placer = typename placers::_NofitPolyPlacer<clppr::Polygon, TBin>;
-    using Selector = selections::_FirstFitSelection<clppr::Polygon>;
+    using Placer = typename placers::_NofitPolyPlacer<ExPolygon, TBin>;
+    using Selector = selections::_FirstFitSelection<ExPolygon>;
     using Packer   = _Nester<Placer, Selector>;
     using PConfig  = typename Packer::PlacementConfig;
     using Distance = TCoord<PointImpl>;
@@ -168,7 +166,7 @@ protected:
     // as it possibly can be but at the same time, it has to provide
     // reasonable results.
     std::tuple<double /*score*/, Box /*farthest point from bin center*/>
-    objfunc(const Item &item, const clppr::IntPoint &bincenter)
+    objfunc(const Item &item, const Point &bincenter)
     {
         const double bin_area = m_bin_area;
         const SpatIndex& spatindex = m_rtree;
@@ -220,12 +218,12 @@ protected:
         
         switch (compute_case) {
         case BIG_ITEM: {
-            const clppr::IntPoint& minc = ibb.minCorner(); // bottom left corner
-            const clppr::IntPoint& maxc = ibb.maxCorner(); // top right corner
+            const Point& minc = ibb.minCorner(); // bottom left corner
+            const Point& maxc = ibb.maxCorner(); // top right corner
 
             // top left and bottom right corners
-            clppr::IntPoint top_left{getX(minc), getY(maxc)};
-            clppr::IntPoint bottom_right{getX(maxc), getY(minc)};
+            Point top_left{getX(minc), getY(maxc)};
+            Point bottom_right{getX(maxc), getY(minc)};
 
             // Now the distance of the gravity center will be calculated to the
             // five anchor points and the smallest will be chosen.
@@ -381,7 +379,7 @@ public:
         });
 
         if (stopcond) m_pck.stopCondition(stopcond);
-        
+
         m_pck.configure(m_pconf);
     }
      
@@ -452,7 +450,7 @@ template<> std::function<double(const Item&)> AutoArranger<Circle>::get_objfn()
 // Specialization for a generalized polygon.
 // Warning: this is unfinished business. It may or may not work.
 template<>
-std::function<double(const Item &)> AutoArranger<clppr::Polygon>::get_objfn()
+std::function<double(const Item &)> AutoArranger<ExPolygon>::get_objfn()
 {
     auto bincenter = sl::boundingBox(m_bin).center();
     return [this, bincenter](const Item &item) {
@@ -474,6 +472,12 @@ template<class S> Radians min_area_boundingbox_rotation(const S &sh)
         .angleToX();
 }
 
+template<class S>
+Radians fit_into_box_rotation(const S &sh, const _Box<TPoint<S>> &box)
+{
+    return fitIntoBoxRotation<S, TCompute<S>, boost::rational<LargeInt>>(sh, box);
+}
+
 template<class BinT> // Arrange for arbitrary bin type
 void _arrange(
         std::vector<Item> &           shapes,
@@ -511,17 +515,26 @@ void _arrange(
     // Use the minimum bounding box rotation as a starting point.
     // TODO: This only works for convex hull. If we ever switch to concave
     // polygon nesting, a convex hull needs to be calculated.
-    if (params.allow_rotations)
-        for (auto &itm : shapes)
+    if (params.allow_rotations) {
+        for (auto &itm : shapes) {
             itm.rotation(min_area_boundingbox_rotation(itm.rawShape()));
 
+            // If the item is too big, try to find a rotation that makes it fit
+            if constexpr (std::is_same_v<BinT, Box>) {
+                auto bb = itm.boundingBox();
+                if (bb.width() >= bin.width() || bb.height() >= bin.height())
+                    itm.rotate(fit_into_box_rotation(itm.transformedShape(), bin));
+            }
+        }
+    }
+
     arranger(inp.begin(), inp.end());
     for (Item &itm : inp) itm.inflate(-infl);
 }
 
 inline Box to_nestbin(const BoundingBox &bb) { return Box{{bb.min(X), bb.min(Y)}, {bb.max(X), bb.max(Y)}};}
 inline Circle to_nestbin(const CircleBed &c) { return Circle({c.center()(0), c.center()(1)}, c.radius()); }
-inline clppr::Polygon to_nestbin(const Polygon &p) { return sl::create<clppr::Polygon>(Slic3rMultiPoint_to_ClipperPath(p)); }
+inline ExPolygon to_nestbin(const Polygon &p) { return ExPolygon{p}; }
 inline Box to_nestbin(const InfiniteBed &bed) { return Box::infinite({bed.center.x(), bed.center.y()}); }
 
 inline coord_t width(const BoundingBox& box) { return box.max.x() - box.min.x(); }
@@ -568,19 +581,12 @@ static void process_arrangeable(const ArrangePolygon &arrpoly,
     const Vec2crd &offs     = arrpoly.translation;
     double         rotation = arrpoly.rotation;
 
-    if (p.is_counter_clockwise()) p.reverse();
-
-    clppr::Polygon clpath(Slic3rMultiPoint_to_ClipperPath(p));
-
     // This fixes:
     // https://github.com/prusa3d/PrusaSlicer/issues/2209
-    if (clpath.Contour.size() < 3)
+    if (p.points.size() < 3)
         return;
 
-    auto firstp = clpath.Contour.front();
-    clpath.Contour.emplace_back(firstp);
-
-    outp.emplace_back(std::move(clpath));
+    outp.emplace_back(std::move(p));
     outp.back().rotation(rotation);
     outp.back().translation({offs.x(), offs.y()});
     outp.back().binId(arrpoly.bed_idx);
@@ -624,7 +630,7 @@ void arrange(ArrangePolygons &      arrangables,
              const BedT &           bed,
              const ArrangeParams &  params)
 {
-    namespace clppr = ClipperLib;
+    namespace clppr = Slic3r::ClipperLib;
     
     std::vector<Item> items, fixeditems;
     items.reserve(arrangables.size());
@@ -643,8 +649,8 @@ void arrange(ArrangePolygons &      arrangables,
     _arrange(items, fixeditems, to_nestbin(bed), params, pri, cfn);
     
     for(size_t i = 0; i < items.size(); ++i) {
-        clppr::IntPoint tr = items[i].translation();
-        arrangables[i].translation = {coord_t(tr.X), coord_t(tr.Y)};
+        Point tr = items[i].translation();
+        arrangables[i].translation = {coord_t(tr.x()), coord_t(tr.y())};
         arrangables[i].rotation    = items[i].rotation();
         arrangables[i].bed_idx     = items[i].binId();
     }
