@@ -1,13 +1,11 @@
 #include "BoundingBox.hpp"
 #include "ExPolygon.hpp"
 #include "Exception.hpp"
-#include "Geometry.hpp"
+#include "Geometry/MedialAxis.hpp"
 #include "Polygon.hpp"
 #include "Line.hpp"
 #include "ClipperUtils.hpp"
 #include "SVG.hpp"
-#include "polypartition.h"
-#include "poly2tri/poly2tri.h"
 #include <algorithm>
 #include <cassert>
 #include <list>
@@ -80,6 +78,13 @@ bool ExPolygon::is_valid() const
     return true;
 }
 
+void ExPolygon::douglas_peucker(double tolerance)
+{
+    this->contour.douglas_peucker(tolerance);
+    for (Polygon &poly : this->holes)
+        poly.douglas_peucker(tolerance);
+}
+
 bool ExPolygon::contains(const Line &line) const
 {
     return this->contains(Polyline(line.a, line.b));
@@ -87,7 +92,7 @@ bool ExPolygon::contains(const Line &line) const
 
 bool ExPolygon::contains(const Polyline &polyline) const
 {
-    return diff_pl((Polylines)polyline, *this).empty();
+    return diff_pl(polyline, *this).empty();
 }
 
 bool ExPolygon::contains(const Polylines &polylines) const
@@ -109,10 +114,11 @@ bool ExPolygon::contains(const Polylines &polylines) const
 
 bool ExPolygon::contains(const Point &point) const
 {
-    if (!this->contour.contains(point)) return false;
-    for (Polygons::const_iterator it = this->holes.begin(); it != this->holes.end(); ++it) {
-        if (it->contains(point)) return false;
-    }
+    if (! this->contour.contains(point))
+        return false;
+    for (const Polygon &hole : this->holes)
+        if (hole.contains(point))
+            return false;
     return true;
 }
 
@@ -132,8 +138,7 @@ ExPolygon::has_boundary_point(const Point &point) const
     return false;
 }
 
-bool
-ExPolygon::overlaps(const ExPolygon &other) const
+bool ExPolygon::overlaps(const ExPolygon &other) const
 {
     #if 0
     BoundingBox bbox = get_extents(other);
@@ -150,6 +155,7 @@ ExPolygon::overlaps(const ExPolygon &other) const
     #endif
     if (! pl_out.empty())
         return true; 
+    //FIXME ExPolygon::overlaps() shall be commutative, it is not!
     return ! other.contour.points.empty() && this->contains_b(other.contour.points.front());
 }
 
@@ -311,284 +317,6 @@ ExPolygon::medial_axis(double max_width, double min_width, Polylines* polylines)
     polylines->insert(polylines->end(), tp.begin(), tp.end());
 }
 
-/*
-void ExPolygon::get_trapezoids(Polygons* polygons) const
-{
-    ExPolygons expp;
-    expp.push_back(*this);
-    boost::polygon::get_trapezoids(*polygons, expp);
-}
-
-void ExPolygon::get_trapezoids(Polygons* polygons, double angle) const
-{
-    ExPolygon clone = *this;
-    clone.rotate(PI/2 - angle, Point(0,0));
-    clone.get_trapezoids(polygons);
-    for (Polygons::iterator polygon = polygons->begin(); polygon != polygons->end(); ++polygon)
-        polygon->rotate(-(PI/2 - angle), Point(0,0));
-}
-*/
-
-// This algorithm may return more trapezoids than necessary
-// (i.e. it may break a single trapezoid in several because
-// other parts of the object have x coordinates in the middle)
-void ExPolygon::get_trapezoids2(Polygons* polygons) const
-{
-    // get all points of this ExPolygon
-    Points pp = *this;
-    
-    // build our bounding box
-    BoundingBox bb(pp);
-    
-    // get all x coordinates
-    std::vector<coord_t> xx;
-    xx.reserve(pp.size());
-    for (Points::const_iterator p = pp.begin(); p != pp.end(); ++p)
-        xx.push_back(p->x());
-    std::sort(xx.begin(), xx.end());
-    
-    // find trapezoids by looping from first to next-to-last coordinate
-    for (std::vector<coord_t>::const_iterator x = xx.begin(); x != xx.end()-1; ++x) {
-        coord_t next_x = *(x + 1);
-        if (*x != next_x)
-            // intersect with rectangle
-            // append results to return value
-            polygons_append(*polygons, intersection({ { { *x, bb.min.y() }, { next_x, bb.min.y() }, { next_x, bb.max.y() }, { *x, bb.max.y() } } }, to_polygons(*this)));
-    }
-}
-
-void ExPolygon::get_trapezoids2(Polygons* polygons, double angle) const
-{
-    ExPolygon clone = *this;
-    clone.rotate(PI/2 - angle, Point(0,0));
-    clone.get_trapezoids2(polygons);
-    for (Polygons::iterator polygon = polygons->begin(); polygon != polygons->end(); ++polygon)
-        polygon->rotate(-(PI/2 - angle), Point(0,0));
-}
-
-// While this triangulates successfully, it's NOT a constrained triangulation
-// as it will create more vertices on the boundaries than the ones supplied.
-void ExPolygon::triangulate(Polygons* polygons) const
-{
-    // first make trapezoids
-    Polygons trapezoids;
-    this->get_trapezoids2(&trapezoids);
-    
-    // then triangulate each trapezoid
-    for (Polygons::iterator polygon = trapezoids.begin(); polygon != trapezoids.end(); ++polygon)
-        polygon->triangulate_convex(polygons);
-}
-
-/*
-void ExPolygon::triangulate_pp(Polygons* polygons) const
-{
-    // convert polygons
-    std::list<TPPLPoly> input;
-    
-    ExPolygons expp = union_ex(simplify_polygons(to_polygons(*this), true));
-    
-    for (ExPolygons::const_iterator ex = expp.begin(); ex != expp.end(); ++ex) {
-        // contour
-        {
-            TPPLPoly p;
-            p.Init(int(ex->contour.points.size()));
-            //printf("%zu\n0\n", ex->contour.points.size());
-            for (const Point &point : ex->contour.points) {
-                size_t i = &point - &ex->contour.points.front();
-                p[i].x = point(0);
-                p[i].y = point(1);
-                //printf("%ld %ld\n", point->x(), point->y());
-            }
-            p.SetHole(false);
-            input.push_back(p);
-        }
-    
-        // holes
-        for (Polygons::const_iterator hole = ex->holes.begin(); hole != ex->holes.end(); ++hole) {
-            TPPLPoly p;
-            p.Init(hole->points.size());
-            //printf("%zu\n1\n", hole->points.size());
-            for (const Point &point : hole->points) {
-                size_t i = &point - &hole->points.front();
-                p[i].x = point(0);
-                p[i].y = point(1);
-                //printf("%ld %ld\n", point->x(), point->y());
-            }
-            p.SetHole(true);
-            input.push_back(p);
-        }
-    }
-    
-    // perform triangulation
-    std::list<TPPLPoly> output;
-    int res = TPPLPartition().Triangulate_MONO(&input, &output);
-    if (res != 1)
-        throw Slic3r::RuntimeError("Triangulation failed");
-    
-    // convert output polygons
-    for (std::list<TPPLPoly>::iterator poly = output.begin(); poly != output.end(); ++poly) {
-        long num_points = poly->GetNumPoints();
-        Polygon p;
-        p.points.resize(num_points);
-        for (long i = 0; i < num_points; ++i) {
-            p.points[i](0) = coord_t((*poly)[i].x);
-            p.points[i](1) = coord_t((*poly)[i].y);
-        }
-        polygons->push_back(p);
-    }
-}
-*/
-
-std::list<TPPLPoly> expoly_to_polypartition_input(const ExPolygon &ex)
-{
-	std::list<TPPLPoly> input;
-	// contour
-	{
-		input.emplace_back();
-		TPPLPoly &p = input.back();
-		p.Init(int(ex.contour.points.size()));
-		for (const Point &point : ex.contour.points) {
-			size_t i = &point - &ex.contour.points.front();
-			p[i].x = point(0);
-			p[i].y = point(1);
-		}
-		p.SetHole(false);
-	}
-	// holes
-	for (const Polygon &hole : ex.holes) {
-		input.emplace_back();
-		TPPLPoly &p = input.back();
-		p.Init(hole.points.size());
-		for (const Point &point : hole.points) {
-			size_t i = &point - &hole.points.front();
-			p[i].x = point(0);
-			p[i].y = point(1);
-		}
-		p.SetHole(true);
-	}
-	return input;
-}
-
-std::list<TPPLPoly> expoly_to_polypartition_input(const ExPolygons &expps)
-{
-    std::list<TPPLPoly> input;
-	for (const ExPolygon &ex : expps) {
-        // contour
-        {
-            input.emplace_back();
-            TPPLPoly &p = input.back();
-            p.Init(int(ex.contour.points.size()));
-            for (const Point &point : ex.contour.points) {
-                size_t i = &point - &ex.contour.points.front();
-                p[i].x = point(0);
-                p[i].y = point(1);
-            }
-            p.SetHole(false);
-        }
-        // holes
-        for (const Polygon &hole : ex.holes) {
-            input.emplace_back();
-            TPPLPoly &p = input.back();
-            p.Init(hole.points.size());
-            for (const Point &point : hole.points) {
-                size_t i = &point - &hole.points.front();
-                p[i].x = point(0);
-                p[i].y = point(1);
-            }
-            p.SetHole(true);
-        }
-    }
-    return input;
-}
-
-std::vector<Point> polypartition_output_to_triangles(const std::list<TPPLPoly> &output)
-{
-    size_t num_triangles = 0;
-    for (const TPPLPoly &poly : output)
-        if (poly.GetNumPoints() >= 3)
-            num_triangles += (size_t)poly.GetNumPoints() - 2;
-    std::vector<Point> triangles;
-    triangles.reserve(triangles.size() + num_triangles * 3);
-    for (const TPPLPoly &poly : output) {
-        long num_points = poly.GetNumPoints();
-        if (num_points >= 3) {
-            const TPPLPoint *pt0 = &poly[0];
-            const TPPLPoint *pt1 = nullptr;
-            const TPPLPoint *pt2 = &poly[1];
-            for (long i = 2; i < num_points; ++ i) {
-                pt1 = pt2;
-                pt2 = &poly[i];
-                triangles.emplace_back(coord_t(pt0->x), coord_t(pt0->y));
-                triangles.emplace_back(coord_t(pt1->x), coord_t(pt1->y));
-                triangles.emplace_back(coord_t(pt2->x), coord_t(pt2->y));
-            }
-        }
-    }
-    return triangles;
-}
-
-void ExPolygon::triangulate_pp(Points *triangles) const
-{
-    ExPolygons expp = union_ex(simplify_polygons(to_polygons(*this), true));
-    std::list<TPPLPoly> input = expoly_to_polypartition_input(expp);
-    // perform triangulation
-    std::list<TPPLPoly> output;
-    int res = TPPLPartition().Triangulate_MONO(&input, &output);
-// int TPPLPartition::Triangulate_EC(TPPLPolyList *inpolys, TPPLPolyList *triangles) {
-    if (res != 1)
-        throw Slic3r::RuntimeError("Triangulation failed");
-    *triangles = polypartition_output_to_triangles(output);
-}
-
-// Uses the Poly2tri library maintained by Jan Niklas Hasse @jhasse // https://github.com/jhasse/poly2tri
-// See https://github.com/jhasse/poly2tri/blob/master/README.md for the limitations of the library!
-// No duplicate points are allowed, no very close points, holes must not touch outer contour etc.
-void ExPolygon::triangulate_p2t(Polygons* polygons) const
-{
-    ExPolygons expp = simplify_polygons_ex(*this, true);
-    
-    for (ExPolygons::const_iterator ex = expp.begin(); ex != expp.end(); ++ex) {
-        // TODO: prevent duplicate points
-
-        // contour
-        std::vector<p2t::Point*> ContourPoints;
-        for (const Point &pt : ex->contour.points)
-            // We should delete each p2t::Point object
-            ContourPoints.push_back(new p2t::Point(pt(0), pt(1)));
-        p2t::CDT cdt(ContourPoints);
-
-        // holes
-        for (Polygons::const_iterator hole = ex->holes.begin(); hole != ex->holes.end(); ++hole) {
-            std::vector<p2t::Point*> points;
-            for (const Point &pt : hole->points)
-                // will be destructed in SweepContext::~SweepContext
-                points.push_back(new p2t::Point(pt(0), pt(1)));
-            cdt.AddHole(points);
-        }
-        
-        // perform triangulation
-        try {
-            cdt.Triangulate();
-            std::vector<p2t::Triangle*> triangles = cdt.GetTriangles();
-            
-            for (std::vector<p2t::Triangle*>::const_iterator triangle = triangles.begin(); triangle != triangles.end(); ++triangle) {
-                Polygon p;
-                for (int i = 0; i <= 2; ++i) {
-                    p2t::Point* point = (*triangle)->GetPoint(i);
-                    p.points.push_back(Point(point->x, point->y));
-                }
-                polygons->push_back(p);
-            }
-        } catch (const Slic3r::RuntimeError & /* err */) {
-            assert(false);
-            // just ignore, don't triangulate
-        }
-
-        for (p2t::Point *ptr : ContourPoints)
-            delete ptr;
-    }
-}
-
 Lines ExPolygon::lines() const
 {
     Lines lines = this->contour.lines();
@@ -640,11 +368,83 @@ extern std::vector<BoundingBox> get_extents_vector(const ExPolygons &polygons)
     return out;
 }
 
+bool has_duplicate_points(const ExPolygon &expoly)
+{
+#if 1
+    // Check globally.
+    size_t cnt = expoly.contour.points.size();
+    for (const Polygon &hole : expoly.holes)
+        cnt += hole.points.size();
+    std::vector<Point> allpts;
+    allpts.reserve(cnt);
+    allpts.insert(allpts.begin(), expoly.contour.points.begin(), expoly.contour.points.end());
+    for (const Polygon &hole : expoly.holes)
+        allpts.insert(allpts.end(), hole.points.begin(), hole.points.end());
+    return has_duplicate_points(std::move(allpts));
+#else
+    // Check per contour.
+    if (has_duplicate_points(expoly.contour))
+        return true;
+    for (const Polygon &hole : expoly.holes)
+        if (has_duplicate_points(hole))
+            return true;
+    return false;
+#endif
+}
+
+bool has_duplicate_points(const ExPolygons &expolys)
+{
+#if 1
+    // Check globally.
+    size_t cnt = 0;
+    for (const ExPolygon &expoly : expolys) {
+        cnt += expoly.contour.points.size();
+        for (const Polygon &hole : expoly.holes)
+            cnt += hole.points.size();
+    }
+    std::vector<Point> allpts;
+    allpts.reserve(cnt);
+    for (const ExPolygon &expoly : expolys) {
+        allpts.insert(allpts.begin(), expoly.contour.points.begin(), expoly.contour.points.end());
+        for (const Polygon &hole : expoly.holes)
+            allpts.insert(allpts.end(), hole.points.begin(), hole.points.end());
+    }
+    return has_duplicate_points(std::move(allpts));
+#else
+    // Check per contour.
+    for (const ExPolygon &expoly : expolys)
+        if (has_duplicate_points(expoly))
+            return true;
+    return false;
+#endif
+}
+
 bool remove_sticks(ExPolygon &poly)
 {
     return remove_sticks(poly.contour) || remove_sticks(poly.holes);
 }
 
+bool remove_small_and_small_holes(ExPolygons &expolygons, double min_area)
+{
+    bool   modified = false;
+    size_t free_idx = 0;
+    for (size_t expoly_idx = 0; expoly_idx < expolygons.size(); ++expoly_idx) {
+        if (std::abs(expolygons[expoly_idx].area()) >= min_area) {
+            // Expolygon is big enough, so also check all its holes
+            modified |= remove_small(expolygons[expoly_idx].holes, min_area);
+            if (free_idx < expoly_idx) {
+                std::swap(expolygons[expoly_idx].contour, expolygons[free_idx].contour);
+                std::swap(expolygons[expoly_idx].holes, expolygons[free_idx].holes);
+            }
+            ++free_idx;
+        } else
+            modified = true;
+    }
+    if (free_idx < expolygons.size())
+        expolygons.erase(expolygons.begin() + free_idx, expolygons.end());
+    return modified;
+}
+
 void keep_largest_contour_only(ExPolygons &polygons)
 {
 	if (polygons.size() > 1) {
