@@ -8,9 +8,12 @@
 #include "../Print.hpp"
 #include "../PrintConfig.hpp"
 #include "../Surface.hpp"
+#include "../PerimeterGenerator.hpp"
 
 #include "FillBase.hpp"
 #include "FillRectilinear.hpp"
+#include "FillLightning.hpp"
+#include "FillConcentric.hpp"
 
 namespace Slic3r {
 
@@ -318,7 +321,7 @@ void export_group_fills_to_svg(const char *path, const std::vector<SurfaceFill>
 #endif
 
 // friend to Layer
-void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive::Octree* support_fill_octree)
+void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive::Octree* support_fill_octree, FillLightning::Generator* lightning_generator)
 {
 	for (LayerRegion *layerm : m_regions)
 		layerm->fills.clear();
@@ -328,9 +331,10 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
 //	this->export_region_fill_surfaces_to_svg_debug("10_fill-initial");
 #endif /* SLIC3R_DEBUG_SLICE_PROCESSING */
 
-	std::vector<SurfaceFill>  surface_fills = group_fills(*this);
-	const Slic3r::BoundingBox bbox 			= this->object()->bounding_box();
-	const auto                resolution 	= this->object()->print()->config().gcode_resolution.value;
+    std::vector<SurfaceFill>  surface_fills       = group_fills(*this);
+    const Slic3r::BoundingBox bbox                = this->object()->bounding_box();
+    const auto                resolution          = this->object()->print()->config().gcode_resolution.value;
+    const auto                perimeter_generator = this->object()->config().perimeter_generator;
 
 #ifdef SLIC3R_DEBUG_SLICE_PROCESSING
 	{
@@ -348,6 +352,16 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
         f->angle 	= surface_fill.params.angle;
         f->adapt_fill_octree = (surface_fill.params.pattern == ipSupportCubic) ? support_fill_octree : adaptive_fill_octree;
 
+        if (surface_fill.params.pattern == ipLightning)
+            dynamic_cast<FillLightning::Filler*>(f.get())->generator = lightning_generator;
+
+        if (perimeter_generator.value == PerimeterGeneratorType::Arachne && surface_fill.params.pattern == ipConcentric) {
+            FillConcentric *fill_concentric = dynamic_cast<FillConcentric *>(f.get());
+            assert(fill_concentric != nullptr);
+            fill_concentric->print_config        = &this->object()->print()->config();
+            fill_concentric->print_object_config = &this->object()->config();
+        }
+
         // calculate flow spacing for infill pattern generation
         bool using_internal_flow = ! surface_fill.surface.is_solid() && ! surface_fill.params.bridge;
         double link_max_length = 0.;
@@ -368,23 +382,29 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
 
         // apply half spacing using this flow's own spacing and generate infill
         FillParams params;
-        params.density 		     = float(0.01 * surface_fill.params.density);
-		params.dont_adjust		 = false; //  surface_fill.params.dont_adjust;
+        params.density           = float(0.01 * surface_fill.params.density);
+        params.dont_adjust       = false; //  surface_fill.params.dont_adjust;
         params.anchor_length     = surface_fill.params.anchor_length;
-		params.anchor_length_max = surface_fill.params.anchor_length_max;
-		params.resolution        = resolution;
+        params.anchor_length_max = surface_fill.params.anchor_length_max;
+        params.resolution        = resolution;
+        params.use_arachne       = perimeter_generator == PerimeterGeneratorType::Arachne && surface_fill.params.pattern == ipConcentric;
+        params.layer_height      = m_regions[surface_fill.region_id]->layer()->height;
 
         for (ExPolygon &expoly : surface_fill.expolygons) {
 			// Spacing is modified by the filler to indicate adjustments. Reset it for each expolygon.
 			f->spacing = surface_fill.params.spacing;
 			surface_fill.surface.expolygon = std::move(expoly);
-			Polylines polylines;
+            Polylines      polylines;
+            ThickPolylines thick_polylines;
 			try {
-				polylines = f->fill_surface(&surface_fill.surface, params);
+                if (params.use_arachne)
+                    thick_polylines = f->fill_surface_arachne(&surface_fill.surface, params);
+                else
+				    polylines = f->fill_surface(&surface_fill.surface, params);
 			} catch (InfillFailedException &) {
 			}
-	        if (! polylines.empty()) {
-		        // calculate actual flow from spacing (which might have been adjusted by the infill
+            if (!polylines.empty() || !thick_polylines.empty()) {
+                // calculate actual flow from spacing (which might have been adjusted by the infill
 		        // pattern generator)
 		        double flow_mm3_per_mm = surface_fill.params.flow.mm3_per_mm();
 		        double flow_width      = surface_fill.params.flow.width();
@@ -402,10 +422,27 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
 		        m_regions[surface_fill.region_id]->fills.entities.push_back(eec = new ExtrusionEntityCollection());
 		        // Only concentric fills are not sorted.
 		        eec->no_sort = f->no_sort();
-		        extrusion_entities_append_paths(
-		            eec->entities, std::move(polylines),
-		            surface_fill.params.extrusion_role,
-		            flow_mm3_per_mm, float(flow_width), surface_fill.params.flow.height());
+                if (params.use_arachne) {
+                    for (const ThickPolyline &thick_polyline : thick_polylines) {
+                        Flow new_flow = surface_fill.params.flow.with_spacing(float(f->spacing));
+
+                        ExtrusionMultiPath multi_path = thick_polyline_to_multi_path(thick_polyline, surface_fill.params.extrusion_role, new_flow, scaled<float>(0.05), float(SCALED_EPSILON));
+                        // Append paths to collection.
+                        if (!multi_path.empty()) {
+                            if (multi_path.paths.front().first_point() == multi_path.paths.back().last_point())
+                                eec->entities.emplace_back(new ExtrusionLoop(std::move(multi_path.paths)));
+                            else
+                                eec->entities.emplace_back(new ExtrusionMultiPath(std::move(multi_path)));
+                        }
+                    }
+
+                    thick_polylines.clear();
+                } else {
+                    extrusion_entities_append_paths(
+                        eec->entities, std::move(polylines),
+                        surface_fill.params.extrusion_role,
+                        flow_mm3_per_mm, float(flow_width), surface_fill.params.flow.height());
+                }
 		    }
 		}
     }
@@ -614,6 +651,7 @@ void Layer::make_ironing()
 			surface_fill.expolygon = std::move(expoly);
 			Polylines polylines;
 			try {
+                assert(!fill_params.use_arachne);
 				polylines = fill.fill_surface(&surface_fill, fill_params);
 			} catch (InfillFailedException &) {
 			}
