@@ -11,6 +11,8 @@
 #include "BoundingBox.hpp"
 #include "LocalesUtils.hpp"
 
+#include <boost/algorithm/string/predicate.hpp>
+
 
 namespace Slic3r
 {
@@ -170,12 +172,18 @@ public:
 			m_gcode += set_format_F(f);
         }
 
+        // Append newline if at least one of X,Y,E,F was changed.
+        // Otherwise, remove the "G1".
+        if (! boost::ends_with(m_gcode, "G1"))
+            m_gcode += "\n";
+        else
+            m_gcode.erase(m_gcode.end()-2, m_gcode.end());
+
         m_current_pos.x() = x;
         m_current_pos.y() = y;
 
 		// Update the elapsed time with a rough estimate.
         m_elapsed_time += ((len == 0.f) ? std::abs(e) : len) / m_current_feedrate * 60.f;
-		m_gcode += "\n";
 		return *this;
 	}
 
@@ -1180,7 +1188,7 @@ WipeTower::ToolChangeResult WipeTower::finish_layer()
 
     // Ask our writer about how much material was consumed.
     // Skip this in case the layer is sparse and config option to not print sparse layers is enabled.
-    if (! m_no_sparse_layers || toolchanges_on_layer)
+    if (! m_no_sparse_layers || toolchanges_on_layer || first_layer)
         if (m_current_tool < m_used_filament_length.size())
             m_used_filament_length[m_current_tool] += writer.get_and_reset_used_filament_length();
 
@@ -1196,7 +1204,7 @@ void WipeTower::plan_toolchange(float z_par, float layer_height_par, unsigned in
 	if (m_plan.empty() || m_plan.back().z + WT_EPSILON < z_par) // if we moved to a new layer, we'll add it to m_plan first
 		m_plan.push_back(WipeTowerInfo(z_par, layer_height_par));
 
-    if (m_first_layer_idx == size_t(-1) && (! m_no_sparse_layers || old_tool != new_tool))
+    if (m_first_layer_idx == size_t(-1) && (! m_no_sparse_layers || old_tool != new_tool || m_plan.size() == 1))
         m_first_layer_idx = m_plan.size() - 1;
 
     if (old_tool == new_tool)	// new layer without toolchanges - we are done
