@@ -47,9 +47,9 @@ void MultiPoint::rotate(double angle, const Point &center)
 
 double MultiPoint::length() const
 {
-    Lines lines = this->lines();
+    const Lines& lines = this->lines();
     double len = 0;
-    for (Lines::iterator it = lines.begin(); it != lines.end(); ++it) {
+    for (auto it = lines.cbegin(); it != lines.cend(); ++it) {
         len += it->length();
     }
     return len;
@@ -63,6 +63,24 @@ int MultiPoint::find_point(const Point &point) const
     return -1;  // not found
 }
 
+int MultiPoint::find_point(const Point &point, double scaled_epsilon) const
+{
+    if (scaled_epsilon == 0)
+        return this->find_point(point);
+
+    auto dist2_min = std::numeric_limits<double>::max();
+    auto eps2      = scaled_epsilon * scaled_epsilon;
+    int  idx_min  = -1;
+    for (const Point &pt : this->points) {
+        double d2 = (pt - point).cast<double>().squaredNorm();
+        if (d2 < dist2_min) {
+            idx_min = int(&pt - &this->points.front());
+            dist2_min = d2;
+        }
+    }
+    return dist2_min < eps2 ? idx_min : -1;
+}
+
 bool MultiPoint::has_boundary_point(const Point &point) const
 {
     double dist = (point.projection_onto(*this) - point).cast<double>().norm();
