@@ -11,16 +11,13 @@
 namespace Slic3r {
 
 class Polygon;
-typedef std::vector<Polygon> Polygons;
+using Polygons          = std::vector<Polygon>;
+using PolygonPtrs       = std::vector<Polygon*>;
+using ConstPolygonPtrs  = std::vector<const Polygon*>;
 
 class Polygon : public MultiPoint
 {
 public:
-    explicit operator Polygons() const { Polygons pp; pp.push_back(*this); return pp; }
-    explicit operator Polyline() const { return this->split_at_first_point(); }
-    Point& operator[](Points::size_type idx) { return this->points[idx]; }
-    const Point& operator[](Points::size_type idx) const { return this->points[idx]; }
-
     Polygon() = default;
     virtual ~Polygon() = default;
     explicit Polygon(const Points &points) : MultiPoint(points) {}
@@ -37,6 +34,9 @@ public:
     Polygon& operator=(const Polygon &other) { points = other.points; return *this; }
     Polygon& operator=(Polygon &&other) { points = std::move(other.points); return *this; }
 
+    Point& operator[](Points::size_type idx) { return this->points[idx]; }
+    const Point& operator[](Points::size_type idx) const { return this->points[idx]; }
+
     // last point == first point for polygons
     const Point& last_point() const override { return this->points.front(); }
 
@@ -55,6 +55,7 @@ public:
     bool make_counter_clockwise();
     bool make_clockwise();
     bool is_valid() const { return this->points.size() >= 3; }
+    void douglas_peucker(double tolerance);
 
     // Does an unoriented polygon contain a point?
     // Tested by counting intersections along a horizontal line.
@@ -69,16 +70,28 @@ public:
     // Projection of a point onto the polygon.
     Point point_projection(const Point &point) const;
     std::vector<float> parameter_by_length() const;
+
+    using iterator = Points::iterator;
+    using const_iterator = Points::const_iterator;
 };
 
 inline bool operator==(const Polygon &lhs, const Polygon &rhs) { return lhs.points == rhs.points; }
 inline bool operator!=(const Polygon &lhs, const Polygon &rhs) { return lhs.points != rhs.points; }
 
-extern BoundingBox get_extents(const Polygon &poly);
-extern BoundingBox get_extents(const Polygons &polygons);
-extern BoundingBox get_extents_rotated(const Polygon &poly, double angle);
-extern BoundingBox get_extents_rotated(const Polygons &polygons, double angle);
-extern std::vector<BoundingBox> get_extents_vector(const Polygons &polygons);
+BoundingBox get_extents(const Polygon &poly);
+BoundingBox get_extents(const Polygons &polygons);
+BoundingBox get_extents_rotated(const Polygon &poly, double angle);
+BoundingBox get_extents_rotated(const Polygons &polygons, double angle);
+std::vector<BoundingBox> get_extents_vector(const Polygons &polygons);
+
+// Polygon must be valid (at least three points), collinear points and duplicate points removed.
+bool        polygon_is_convex(const Points &poly);
+inline bool polygon_is_convex(const Polygon &poly) { return polygon_is_convex(poly.points); }
+
+// Test for duplicate points. The points are copied, sorted and checked for duplicates globally.
+inline bool has_duplicate_points(Polygon &&poly)      { return has_duplicate_points(std::move(poly.points)); }
+inline bool has_duplicate_points(const Polygon &poly) { return has_duplicate_points(poly.points); }
+bool        has_duplicate_points(const Polygons &polys);
 
 inline double total_length(const Polygons &polylines) {
     double total = 0;
@@ -87,6 +100,8 @@ inline double total_length(const Polygons &polylines) {
     return total;
 }
 
+inline double area(const Polygon &poly) { return poly.area(); }
+
 inline double area(const Polygons &polys)
 {
     double s = 0.;
@@ -96,19 +111,19 @@ inline double area(const Polygons &polys)
 }
 
 // Remove sticks (tentacles with zero area) from the polygon.
-extern bool        remove_sticks(Polygon &poly);
-extern bool        remove_sticks(Polygons &polys);
+bool remove_sticks(Polygon &poly);
+bool remove_sticks(Polygons &polys);
 
 // Remove polygons with less than 3 edges.
-extern bool        remove_degenerate(Polygons &polys);
-extern bool        remove_small(Polygons &polys, double min_area);
-extern void 	   remove_collinear(Polygon &poly);
-extern void 	   remove_collinear(Polygons &polys);
+bool remove_degenerate(Polygons &polys);
+bool remove_small(Polygons &polys, double min_area);
+void remove_collinear(Polygon &poly);
+void remove_collinear(Polygons &polys);
 
 // Append a vector of polygons at the end of another vector of polygons.
-inline void        polygons_append(Polygons &dst, const Polygons &src) { dst.insert(dst.end(), src.begin(), src.end()); }
+inline void polygons_append(Polygons &dst, const Polygons &src) { dst.insert(dst.end(), src.begin(), src.end()); }
 
-inline void        polygons_append(Polygons &dst, Polygons &&src) 
+inline void polygons_append(Polygons &dst, Polygons &&src) 
 {
     if (dst.empty()) {
         dst = std::move(src);
@@ -214,6 +229,24 @@ inline Polylines to_polylines(Polygons &&polys)
     return polylines;
 }
 
+inline Polygons to_polygons(const std::vector<Points> &paths)
+{
+    Polygons out;
+    out.reserve(paths.size());
+    for (const Points &path : paths)
+        out.emplace_back(path);
+    return out;
+}
+
+inline Polygons to_polygons(std::vector<Points> &&paths)
+{
+    Polygons out;
+    out.reserve(paths.size());
+    for (const Points &path : paths)
+        out.emplace_back(std::move(path));
+    return out;
+}
+
 } // Slic3r
 
 // start Boost
