@@ -132,37 +132,38 @@ template void Polyline::simplify_by_visibility<ExPolygonCollection>(const ExPoly
 
 void Polyline::split_at(const Point &point, Polyline* p1, Polyline* p2) const
 {
-    if (this->points.empty()) return;
-    
-    // find the line to split at
-    size_t line_idx = 0;
-    Point p = this->first_point();
-    double min = (p - point).cast<double>().norm();
-    Lines lines = this->lines();
-    for (Lines::const_iterator line = lines.begin(); line != lines.end(); ++line) {
-        Point p_tmp = point.projection_onto(*line);
-        if ((p_tmp - point).cast<double>().norm() < min) {
-	        p = p_tmp;
-	        min = (p - point).cast<double>().norm();
-	        line_idx = line - lines.begin();
+    if (this->size() < 2) {
+        *p1 = *this;
+        p2->clear();
+        return;
+    }
+
+    if (this->points.front() == point) {
+        *p1 = { point };
+        *p2 = *this;
+    }
+
+    auto  min_dist2    = std::numeric_limits<double>::max();
+    auto  min_point_it = this->points.cbegin();
+    Point prev         = this->points.front();
+    for (auto it = this->points.cbegin() + 1; it != this->points.cend(); ++ it) {
+        Point proj = point.projection_onto(Line(prev, *it));
+        auto  d2 = (proj - point).cast<double>().squaredNorm();
+        if (d2 < min_dist2) {
+	        min_dist2    = d2;
+	        min_point_it = it;
         }
+        prev = *it;
     }
+
+    p1->points.assign(this->points.cbegin(), min_point_it);
+    if (p1->points.back() != point)
+        p1->points.emplace_back(point);
     
-    // create first half
-    p1->points.clear();
-    for (Lines::const_iterator line = lines.begin(); line != lines.begin() + line_idx + 1; ++line)
-        if (line->a != p) 
-            p1->points.push_back(line->a);
-    // we add point instead of p because they might differ because of numerical issues
-    // and caller might want to rely on point belonging to result polylines
-    p1->points.push_back(point);
-    
-    // create second half
-    p2->points.clear();
-    p2->points.push_back(point);
-    for (Lines::const_iterator line = lines.begin() + line_idx; line != lines.end(); ++line) {
-        p2->points.push_back(line->b);
-    }
+    p2->points = { point };
+    if (*min_point_it == point)
+        ++ min_point_it;
+    p2->points.insert(p2->points.end(), min_point_it, this->points.cend());
 }
 
 bool Polyline::is_straight() const
@@ -224,6 +225,29 @@ bool remove_degenerate(Polylines &polylines)
     return modified;
 }
 
+std::pair<int, Point> foot_pt(const Points &polyline, const Point &pt)
+{
+    if (polyline.size() < 2)
+        return std::make_pair(-1, Point(0, 0));
+
+    auto  d2_min  = std::numeric_limits<double>::max();
+    Point foot_pt_min;
+    Point prev = polyline.front();
+    auto  it = polyline.begin();
+    auto  it_proj = polyline.begin();
+    for (++ it; it != polyline.end(); ++ it) {
+        Point  foot_pt = pt.projection_onto(Line(prev, *it));
+        double d2 = (foot_pt - pt).cast<double>().squaredNorm();
+        if (d2 < d2_min) {
+            d2_min      = d2;
+            foot_pt_min = foot_pt;
+            it_proj     = it;
+        }
+        prev = *it;
+    }
+    return std::make_pair(int(it_proj - polyline.begin()) - 1, foot_pt_min);
+}
+
 ThickLines ThickPolyline::thicklines() const
 {
     ThickLines lines;
@@ -235,6 +259,34 @@ ThickLines ThickPolyline::thicklines() const
     return lines;
 }
 
+// Removes the given distance from the end of the ThickPolyline
+void ThickPolyline::clip_end(double distance)
+{
+    while (distance > 0) {
+        Vec2d    last_point = this->last_point().cast<double>();
+        coordf_t last_width = this->width.back();
+        this->points.pop_back();
+        this->width.pop_back();
+        if (this->points.empty())
+            break;
+
+        Vec2d    vec            = this->last_point().cast<double>() - last_point;
+        coordf_t width_diff     = this->width.back() - last_width;
+        double   vec_length_sqr = vec.squaredNorm();
+        if (vec_length_sqr > distance * distance) {
+            double t = (distance / std::sqrt(vec_length_sqr));
+            this->points.emplace_back((last_point + vec * t).cast<coord_t>());
+            this->width.emplace_back(last_width + width_diff * t);
+            assert(this->width.size() == (this->points.size() - 1) * 2);
+            return;
+        } else
+            this->width.pop_back();
+
+        distance -= std::sqrt(vec_length_sqr);
+    }
+    assert(this->width.size() == (this->points.size() - 1) * 2);
+}
+
 Lines3 Polyline3::lines() const
 {
     Lines3 lines;
