@@ -64,7 +64,7 @@ public:
     const Point& leftmost_point() const;
     Lines lines() const override;
 
-    void clip_end(double distance);
+    virtual void clip_end(double distance);
     void clip_start(double distance);
     void extend_end(double distance);
     void extend_start(double distance);
@@ -162,6 +162,9 @@ const Point& leftmost_point(const Polylines &polylines);
 
 bool remove_degenerate(Polylines &polylines);
 
+// Returns index of a segment of a polyline and foot point of pt on polyline.
+std::pair<int, Point> foot_pt(const Points &polyline, const Point &pt);
+
 class ThickPolyline : public Polyline {
 public:
     ThickPolyline() : endpoints(std::make_pair(false, false)) {}
@@ -172,10 +175,24 @@ public:
         std::swap(this->endpoints.first, this->endpoints.second);
     }
 
+    void clip_end(double distance) override;
+
     std::vector<coordf_t> width;
     std::pair<bool,bool>  endpoints;
 };
 
+inline ThickPolylines to_thick_polylines(Polylines &&polylines, const coordf_t width)
+{
+    ThickPolylines out;
+    out.reserve(polylines.size());
+    for (Polyline &polyline : polylines) {
+        out.emplace_back();
+        out.back().width.assign((polyline.points.size() - 1) * 2, width);
+        out.back().points = std::move(polyline.points);
+    }
+    return out;
+}
+
 class Polyline3 : public MultiPoint3
 {
 public:
