@@ -2,14 +2,12 @@
 #define slic3r_Tesselate_hpp_
 
 #include <vector>
+#include <admesh/stl.h>
 
-#include "Point.hpp"
+#include "ExPolygon.hpp"
 
 namespace Slic3r {
 
-class ExPolygon;
-typedef std::vector<ExPolygon> ExPolygons;
-
 const bool constexpr NORMALS_UP = false;
 const bool constexpr NORMALS_DOWN = true;
 
@@ -20,6 +18,10 @@ extern std::vector<Vec2d> triangulate_expolygons_2d(const ExPolygons &polys, boo
 extern std::vector<Vec2f> triangulate_expolygon_2f (const ExPolygon  &poly,  bool flip = NORMALS_UP);
 extern std::vector<Vec2f> triangulate_expolygons_2f(const ExPolygons &polys, bool flip = NORMALS_UP);
 
+indexed_triangle_set wall_strip(const Polygon &poly,
+                                double         lower_z_mm,
+                                double         upper_z_mm);
+
 } // namespace Slic3r
 
 #endif /* slic3r_Tesselate_hpp_ */
