@@ -5,12 +5,12 @@
 #include "libslic3r/Polygon.hpp"
 #include "libslic3r/ClipperUtils.hpp"
 #include "libslic3r/BoundingBox.hpp"
-#include "libslic3r/Geometry.hpp"
+#include "libslic3r/Geometry/Circle.hpp"
+#include "libslic3r/Tesselate.hpp"
+#include "libslic3r/PresetBundle.hpp"
 
 #include "GUI_App.hpp"
-#include "libslic3r/PresetBundle.hpp"
 #include "GLCanvas3D.hpp"
-#include "3DScene.hpp"
 
 #include <GL/glew.h>
 
@@ -19,63 +19,42 @@
 #include <boost/log/trivial.hpp>
 
 static const float GROUND_Z = -0.02f;
+static const std::array<float, 4> DEFAULT_MODEL_COLOR = { 0.235f, 0.235f, 0.235f, 1.0f };
+static const std::array<float, 4> PICKING_MODEL_COLOR = { 0.0f, 0.0f, 0.0f, 1.0f };
 
 namespace Slic3r {
 namespace GUI {
 
-bool GeometryBuffer::set_from_triangles(const Polygons& triangles, float z, bool generate_tex_coords)
+bool GeometryBuffer::set_from_triangles(const std::vector<Vec2f> &triangles, float z)
 {
-    m_vertices.clear();
-    unsigned int v_size = 3 * (unsigned int)triangles.size();
-
-    if (v_size == 0)
+    if (triangles.empty()) {
+        m_vertices.clear();
         return false;
-
-    m_vertices = std::vector<Vertex>(v_size, Vertex());
-
-    float min_x = unscale<float>(triangles[0].points[0](0));
-    float min_y = unscale<float>(triangles[0].points[0](1));
-    float max_x = min_x;
-    float max_y = min_y;
-
-    unsigned int v_count = 0;
-    for (const Polygon& t : triangles) {
-        for (unsigned int i = 0; i < 3; ++i) {
-            Vertex& v = m_vertices[v_count];
-
-            const Point& p = t.points[i];
-            float x = unscale<float>(p(0));
-            float y = unscale<float>(p(1));
-
-            v.position[0] = x;
-            v.position[1] = y;
-            v.position[2] = z;
-
-            if (generate_tex_coords) {
-                v.tex_coords[0] = x;
-                v.tex_coords[1] = y;
-
-                min_x = std::min(min_x, x);
-                max_x = std::max(max_x, x);
-                min_y = std::min(min_y, y);
-                max_y = std::max(max_y, y);
-            }
-
-            ++v_count;
-        }
     }
 
-    if (generate_tex_coords) {
-        float size_x = max_x - min_x;
-        float size_y = max_y - min_y;
+    assert(triangles.size() % 3 == 0);
+    m_vertices = std::vector<Vertex>(triangles.size(), Vertex());
 
-        if ((size_x != 0.0f) && (size_y != 0.0f)) {
-            float inv_size_x = 1.0f / size_x;
-            float inv_size_y = -1.0f / size_y;
-            for (Vertex& v : m_vertices) {
-                v.tex_coords[0] = (v.tex_coords[0] - min_x) * inv_size_x;
-                v.tex_coords[1] = (v.tex_coords[1] - min_y) * inv_size_y;
-            }
+    Vec2f min = triangles.front();
+    Vec2f max = min;
+
+    for (size_t v_count = 0; v_count < triangles.size(); ++ v_count) {
+        const Vec2f &p = triangles[v_count];
+        Vertex      &v = m_vertices[v_count];
+        v.position   = Vec3f(p.x(), p.y(), z);
+        v.tex_coords = p;
+        min = min.cwiseMin(p).eval();
+        max = max.cwiseMax(p).eval();
+    }
+
+    Vec2f size = max - min;
+    if (size.x() != 0.f && size.y() != 0.f) {
+        Vec2f inv_size = size.cwiseInverse();
+        inv_size.y() *= -1;
+        for (Vertex& v : m_vertices) {
+            v.tex_coords -= min;
+            v.tex_coords.x() *= inv_size.x();
+            v.tex_coords.y() *= inv_size.y();
         }
     }
 
@@ -120,12 +99,6 @@ const float Bed3D::Axes::DefaultStemLength = 25.0f;
 const float Bed3D::Axes::DefaultTipRadius = 2.5f * Bed3D::Axes::DefaultStemRadius;
 const float Bed3D::Axes::DefaultTipLength = 5.0f;
 
-void Bed3D::Axes::set_stem_length(float length)
-{
-    m_stem_length = length;
-    m_arrow.reset();
-}
-
 void Bed3D::Axes::render() const
 {
     auto render_axis = [this](const Transform3f& transform) {
@@ -135,7 +108,8 @@ void Bed3D::Axes::render() const
         glsafe(::glPopMatrix());
     };
 
-    m_arrow.init_from(stilized_arrow(16, DefaultTipRadius, DefaultTipLength, DefaultStemRadius, m_stem_length));
+    if (!m_arrow.is_initialized())
+        const_cast<GLModel*>(&m_arrow)->init_from(stilized_arrow(16, DefaultTipRadius, DefaultTipLength, DefaultStemRadius, m_stem_length));
 
     GLShaderProgram* shader = wxGetApp().get_shader("gouraud_light");
     if (shader == nullptr)
@@ -144,20 +118,18 @@ void Bed3D::Axes::render() const
     glsafe(::glEnable(GL_DEPTH_TEST));
 
     shader->start_using();
+    shader->set_uniform("emission_factor", 0.0f);
 
     // x axis
-    std::array<float, 4> color = { 0.75f, 0.0f, 0.0f, 1.0f };
-    shader->set_uniform("uniform_color", color);
-    render_axis(Geometry::assemble_transform(m_origin, { 0.0, 0.5 * M_PI, 0.0f }).cast<float>());
+    const_cast<GLModel*>(&m_arrow)->set_color(-1, { 0.75f, 0.0f, 0.0f, 1.0f });
+    render_axis(Geometry::assemble_transform(m_origin, { 0.0, 0.5 * M_PI, 0.0 }).cast<float>());
 
     // y axis
-    color = { 0.0f, 0.75f, 0.0f, 1.0f };
-    shader->set_uniform("uniform_color", color);
-    render_axis(Geometry::assemble_transform(m_origin, { -0.5 * M_PI, 0.0, 0.0f }).cast<float>());
+    const_cast<GLModel*>(&m_arrow)->set_color(-1, { 0.0f, 0.75f, 0.0f, 1.0f });
+    render_axis(Geometry::assemble_transform(m_origin, { -0.5 * M_PI, 0.0, 0.0 }).cast<float>());
 
     // z axis
-    color = { 0.0f, 0.0f, 0.75f, 1.0f };
-    shader->set_uniform("uniform_color", color);
+    const_cast<GLModel*>(&m_arrow)->set_color(-1, { 0.0f, 0.0f, 0.75f, 1.0f });
     render_axis(Geometry::assemble_transform(m_origin).cast<float>());
 
     shader->stop_using();
@@ -165,73 +137,69 @@ void Bed3D::Axes::render() const
     glsafe(::glDisable(GL_DEPTH_TEST));
 }
 
-Bed3D::Bed3D()
-    : m_type(Custom)
-    , m_vbo_id(0)
-    , m_scale_factor(1.0f)
-{
-}
-
-bool Bed3D::set_shape(const Pointfs& shape, const std::string& custom_texture, const std::string& custom_model, bool force_as_custom)
+bool Bed3D::set_shape(const Pointfs& bed_shape, const double max_print_height, const std::string& custom_texture, const std::string& custom_model, bool force_as_custom)
 {
     auto check_texture = [](const std::string& texture) {
-        return !texture.empty() && (boost::algorithm::iends_with(texture, ".png") || boost::algorithm::iends_with(texture, ".svg")) && boost::filesystem::exists(texture);
+        boost::system::error_code ec; // so the exists call does not throw (e.g. after a permission problem)
+        return !texture.empty() && (boost::algorithm::iends_with(texture, ".png") || boost::algorithm::iends_with(texture, ".svg")) && boost::filesystem::exists(texture, ec);
     };
 
     auto check_model = [](const std::string& model) {
-        return !model.empty() && boost::algorithm::iends_with(model, ".stl") && boost::filesystem::exists(model);
+        boost::system::error_code ec;
+        return !model.empty() && boost::algorithm::iends_with(model, ".stl") && boost::filesystem::exists(model, ec);
     };
 
-    EType type;
+    Type type;
     std::string model;
     std::string texture;
     if (force_as_custom)
-        type = Custom;
+        type = Type::Custom;
     else {
-        auto [new_type, system_model, system_texture] = detect_type(shape);
+        auto [new_type, system_model, system_texture] = detect_type(bed_shape);
         type = new_type;
         model = system_model;
         texture = system_texture;
     }
 
     std::string texture_filename = custom_texture.empty() ? texture : custom_texture;
-    if (!check_texture(texture_filename))
+    if (! texture_filename.empty() && ! check_texture(texture_filename)) {
+        BOOST_LOG_TRIVIAL(error) << "Unable to load bed texture: " << texture_filename;
         texture_filename.clear();
+    }
 
     std::string model_filename = custom_model.empty() ? model : custom_model;
-    if (!check_model(model_filename))
+    if (! model_filename.empty() && ! check_model(model_filename)) {
+        BOOST_LOG_TRIVIAL(error) << "Unable to load bed model: " << model_filename;
         model_filename.clear();
+    }
 
-    if (m_shape == shape && m_type == type && m_texture_filename == texture_filename && m_model_filename == model_filename)
+    
+    if (m_build_volume.bed_shape() == bed_shape && m_build_volume.max_print_height() == max_print_height && m_type == type && m_texture_filename == texture_filename && m_model_filename == model_filename)
         // No change, no need to update the UI.
         return false;
 
-    m_shape = shape;
+    m_type = type;
+    m_build_volume = BuildVolume { bed_shape, max_print_height };
     m_texture_filename = texture_filename;
     m_model_filename = model_filename;
-    m_type = type;
+    m_extended_bounding_box = this->calc_extended_bounding_box();
 
-    calc_bounding_boxes();
-
-    ExPolygon poly;
-    for (const Vec2d& p : m_shape) {
-        poly.contour.append(Point(scale_(p(0)), scale_(p(1))));
-    }
+    ExPolygon poly{ Polygon::new_scale(bed_shape) };
 
     calc_triangles(poly);
 
     const BoundingBox& bed_bbox = poly.contour.bounding_box();
     calc_gridlines(poly, bed_bbox);
 
-    m_polygon = offset_ex(poly.contour, (float)bed_bbox.radius() * 1.7f, jtRound, scale_(0.5))[0].contour;
+    m_polygon = offset(poly.contour, (float)bed_bbox.radius() * 1.7f, jtRound, scale_(0.5))[0];
 
-    reset();
+    this->release_VBOs();
     m_texture.reset();
     m_model.reset();
 
     // Set the origin and size for rendering the coordinate system axes.
     m_axes.set_origin({ 0.0, 0.0, static_cast<double>(GROUND_Z) });
-    m_axes.set_stem_length(0.1f * static_cast<float>(m_bounding_box.max_size()));
+    m_axes.set_stem_length(0.1f * static_cast<float>(m_build_volume.bounding_volume().max_size()));
 
     // Let the calee to update the UI.
     return true;
@@ -247,69 +215,81 @@ Point Bed3D::point_projection(const Point& point) const
     return m_polygon.point_projection(point);
 }
 
-void Bed3D::render(GLCanvas3D& canvas, bool bottom, float scale_factor,
-                   bool show_axes, bool show_texture) const
+void Bed3D::render(GLCanvas3D& canvas, bool bottom, float scale_factor, bool show_axes, bool show_texture)
 {
-    m_scale_factor = scale_factor;
+    render_internal(canvas, bottom, scale_factor, show_axes, show_texture, false);
+}
+
+void Bed3D::render_for_picking(GLCanvas3D& canvas, bool bottom, float scale_factor)
+{
+    render_internal(canvas, bottom, scale_factor, false, false, true);
+}
+
+void Bed3D::render_internal(GLCanvas3D& canvas, bool bottom, float scale_factor,
+    bool show_axes, bool show_texture, bool picking)
+{
+    float* factor = const_cast<float*>(&m_scale_factor);
+    *factor = scale_factor;
 
     if (show_axes)
         render_axes();
 
     glsafe(::glEnable(GL_DEPTH_TEST));
 
+    m_model.set_color(-1, picking ? PICKING_MODEL_COLOR : DEFAULT_MODEL_COLOR);
+
     switch (m_type)
     {
-    case System: { render_system(canvas, bottom, show_texture); break; }
+    case Type::System: { render_system(canvas, bottom, show_texture); break; }
     default:
-    case Custom: { render_custom(canvas, bottom, show_texture); break; }
+    case Type::Custom: { render_custom(canvas, bottom, show_texture, picking); break; }
     }
 
     glsafe(::glDisable(GL_DEPTH_TEST));
 }
 
-void Bed3D::calc_bounding_boxes() const
+// Calculate an extended bounding box from axes and current model for visualization purposes.
+BoundingBoxf3 Bed3D::calc_extended_bounding_box() const
 {
-    m_bounding_box = BoundingBoxf3();
-    for (const Vec2d& p : m_shape) {
-        m_bounding_box.merge(Vec3d(p(0), p(1), 0.0));
-    }
-
-    m_extended_bounding_box = m_bounding_box;
-
+    BoundingBoxf3 out { m_build_volume.bounding_volume() };
+    const Vec3d size = out.size();
+    // ensures that the bounding box is set as defined or the following calls to merge() will not work as intented
+    if (size.x() > 0.0 && size.y() > 0.0 && !out.defined)
+        out.defined = true;
+    // Reset the build volume Z, we don't want to zoom to the top of the build volume if it is empty.
+    out.min.z() = 0.0;
+    out.max.z() = 0.0;
     // extend to contain axes
-    m_extended_bounding_box.merge(m_axes.get_origin() + m_axes.get_total_length() * Vec3d::Ones());
-    m_extended_bounding_box.merge(m_extended_bounding_box.min + Vec3d(-Axes::DefaultTipRadius, -Axes::DefaultTipRadius, m_extended_bounding_box.max(2)));
-
+    out.merge(m_axes.get_origin() + m_axes.get_total_length() * Vec3d::Ones());
+    out.merge(out.min + Vec3d(-Axes::DefaultTipRadius, -Axes::DefaultTipRadius, out.max.z()));
     // extend to contain model, if any
     BoundingBoxf3 model_bb = m_model.get_bounding_box();
     if (model_bb.defined) {
         model_bb.translate(m_model_offset);
-        m_extended_bounding_box.merge(model_bb);
+        out.merge(model_bb);
     }
+    return out;
 }
 
 void Bed3D::calc_triangles(const ExPolygon& poly)
 {
-    Polygons triangles;
-    poly.triangulate_p2t(&triangles);
-
-    if (!m_triangles.set_from_triangles(triangles, GROUND_Z, true))
-        printf("Unable to create bed triangles\n");
+    if (! m_triangles.set_from_triangles(triangulate_expolygon_2f(poly, NORMALS_UP), GROUND_Z))
+        BOOST_LOG_TRIVIAL(error) << "Unable to create bed triangles";
 }
 
 void Bed3D::calc_gridlines(const ExPolygon& poly, const BoundingBox& bed_bbox)
 {
     Polylines axes_lines;
-    for (coord_t x = bed_bbox.min(0); x <= bed_bbox.max(0); x += scale_(10.0)) {
+    for (coord_t x = bed_bbox.min.x(); x <= bed_bbox.max.x(); x += scale_(10.0)) {
         Polyline line;
-        line.append(Point(x, bed_bbox.min(1)));
-        line.append(Point(x, bed_bbox.max(1)));
+        line.append(Point(x, bed_bbox.min.y()));
+        line.append(Point(x, bed_bbox.max.y()));
         axes_lines.push_back(line);
     }
-    for (coord_t y = bed_bbox.min(1); y <= bed_bbox.max(1); y += scale_(10.0)) {
+    for (coord_t y = bed_bbox.min.y(); y <= bed_bbox.max.y(); y += scale_(10.0)) {
         Polyline line;
-        line.append(Point(bed_bbox.min(0), y));
-        line.append(Point(bed_bbox.max(0), y));
+        line.append(Point(bed_bbox.min.x(), y));
+        line.append(Point(bed_bbox.max.x(), y));
         axes_lines.push_back(line);
     }
 
@@ -321,11 +301,12 @@ void Bed3D::calc_gridlines(const ExPolygon& poly, const BoundingBox& bed_bbox)
     std::copy(contour_lines.begin(), contour_lines.end(), std::back_inserter(gridlines));
 
     if (!m_gridlines.set_from_lines(gridlines, GROUND_Z))
-        printf("Unable to create bed grid lines\n");
+        BOOST_LOG_TRIVIAL(error) << "Unable to create bed grid lines\n";
 }
 
-
-std::tuple<Bed3D::EType, std::string, std::string> Bed3D::detect_type(const Pointfs& shape) const
+// Try to match the print bed shape with the shape of an active profile. If such a match exists,
+// return the print bed model.
+std::tuple<Bed3D::Type, std::string, std::string> Bed3D::detect_type(const Pointfs& shape)
 {
     auto bundle = wxGetApp().preset_bundle;
     if (bundle != nullptr) {
@@ -336,7 +317,7 @@ std::tuple<Bed3D::EType, std::string, std::string> Bed3D::detect_type(const Poin
                     std::string model_filename = PresetUtils::system_printer_bed_model(*curr);
                     std::string texture_filename = PresetUtils::system_printer_bed_texture(*curr);
                     if (!model_filename.empty() && !texture_filename.empty())
-                        return { System, model_filename, texture_filename };
+                        return { Type::System, model_filename, texture_filename };
                 }
             }
 
@@ -344,12 +325,12 @@ std::tuple<Bed3D::EType, std::string, std::string> Bed3D::detect_type(const Poin
         }
     }
 
-    return { Custom, "", "" };
+    return { Type::Custom, {}, {} };
 }
 
 void Bed3D::render_axes() const
 {
-    if (!m_shape.empty())
+    if (m_build_volume.valid())
         m_axes.render();
 }
 
@@ -364,64 +345,66 @@ void Bed3D::render_system(GLCanvas3D& canvas, bool bottom, bool show_texture) co
 
 void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
 {
+    GLTexture* texture = const_cast<GLTexture*>(&m_texture);
+    GLTexture* temp_texture = const_cast<GLTexture*>(&m_temp_texture);
+
     if (m_texture_filename.empty()) {
-        m_texture.reset();
-        render_default(bottom);
+        texture->reset();
+        render_default(bottom, false);
         return;
     }
 
-    if ((m_texture.get_id() == 0) || (m_texture.get_source() != m_texture_filename)) {
-        m_texture.reset();
+    if (texture->get_id() == 0 || texture->get_source() != m_texture_filename) {
+        texture->reset();
 
         if (boost::algorithm::iends_with(m_texture_filename, ".svg")) {
             // use higher resolution images if graphic card and opengl version allow
             GLint max_tex_size = OpenGLManager::get_gl_info().get_max_tex_size();
-            if ((m_temp_texture.get_id() == 0) || (m_temp_texture.get_source() != m_texture_filename)) {
+            if (temp_texture->get_id() == 0 || temp_texture->get_source() != m_texture_filename) {
                 // generate a temporary lower resolution texture to show while no main texture levels have been compressed
-                if (!m_temp_texture.load_from_svg_file(m_texture_filename, false, false, false, max_tex_size / 8)) {
-                    render_default(bottom);
+                if (!temp_texture->load_from_svg_file(m_texture_filename, false, false, false, max_tex_size / 8)) {
+                    render_default(bottom, false);
                     return;
                 }
                 canvas.request_extra_frame();
             }
 
             // starts generating the main texture, compression will run asynchronously
-            if (!m_texture.load_from_svg_file(m_texture_filename, true, true, true, max_tex_size)) {
-                render_default(bottom);
+            if (!texture->load_from_svg_file(m_texture_filename, true, true, true, max_tex_size)) {
+                render_default(bottom, false);
                 return;
             }
         } 
         else if (boost::algorithm::iends_with(m_texture_filename, ".png")) {
             // generate a temporary lower resolution texture to show while no main texture levels have been compressed
-            if ((m_temp_texture.get_id() == 0) || (m_temp_texture.get_source() != m_texture_filename)) {
-                if (!m_temp_texture.load_from_file(m_texture_filename, false, GLTexture::None, false)) {
-                    render_default(bottom);
+            if (temp_texture->get_id() == 0 || temp_texture->get_source() != m_texture_filename) {
+                if (!temp_texture->load_from_file(m_texture_filename, false, GLTexture::None, false)) {
+                    render_default(bottom, false);
                     return;
                 }
                 canvas.request_extra_frame();
             }
 
             // starts generating the main texture, compression will run asynchronously
-            if (!m_texture.load_from_file(m_texture_filename, true, GLTexture::MultiThreaded, true)) {
-                render_default(bottom);
+            if (!texture->load_from_file(m_texture_filename, true, GLTexture::MultiThreaded, true)) {
+                render_default(bottom, false);
                 return;
             }
         }
         else {
-            render_default(bottom);
+            render_default(bottom, false);
             return;
         }
     }
-    else if (m_texture.unsent_compressed_data_available()) {
+    else if (texture->unsent_compressed_data_available()) {
         // sends to gpu the already available compressed levels of the main texture
-        m_texture.send_compressed_data_to_gpu();
+        texture->send_compressed_data_to_gpu();
 
         // the temporary texture is not needed anymore, reset it
-        if (m_temp_texture.get_id() != 0)
-            m_temp_texture.reset();
+        if (temp_texture->get_id() != 0)
+            temp_texture->reset();
 
         canvas.request_extra_frame();
-
     }
 
     if (m_triangles.get_vertices_count() > 0) {
@@ -431,15 +414,18 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
             shader->set_uniform("transparent_background", bottom);
             shader->set_uniform("svg_source", boost::algorithm::iends_with(m_texture.get_source(), ".svg"));
 
-            if (m_vbo_id == 0) {
-                glsafe(::glGenBuffers(1, &m_vbo_id));
-                glsafe(::glBindBuffer(GL_ARRAY_BUFFER, m_vbo_id));
+            unsigned int* vbo_id = const_cast<unsigned int*>(&m_vbo_id);
+
+            if (*vbo_id == 0) {
+                glsafe(::glGenBuffers(1, vbo_id));
+                glsafe(::glBindBuffer(GL_ARRAY_BUFFER, *vbo_id));
                 glsafe(::glBufferData(GL_ARRAY_BUFFER, (GLsizeiptr)m_triangles.get_vertices_data_size(), (const GLvoid*)m_triangles.get_vertices_data(), GL_STATIC_DRAW));
                 glsafe(::glBindBuffer(GL_ARRAY_BUFFER, 0));
             }
 
             glsafe(::glEnable(GL_DEPTH_TEST));
-            glsafe(::glDepthMask(GL_FALSE));
+            if (bottom)
+                glsafe(::glDepthMask(GL_FALSE));
 
             glsafe(::glEnable(GL_BLEND));
             glsafe(::glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));
@@ -453,12 +439,12 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
             GLint tex_coords_id = shader->get_attrib_location("v_tex_coords");
 
             // show the temporary texture while no compressed data is available
-            GLuint tex_id = (GLuint)m_temp_texture.get_id();
+            GLuint tex_id = (GLuint)temp_texture->get_id();
             if (tex_id == 0)
-                tex_id = (GLuint)m_texture.get_id();
+                tex_id = (GLuint)texture->get_id();
 
             glsafe(::glBindTexture(GL_TEXTURE_2D, tex_id));
-            glsafe(::glBindBuffer(GL_ARRAY_BUFFER, m_vbo_id));
+            glsafe(::glBindBuffer(GL_ARRAY_BUFFER, *vbo_id));
 
             if (position_id != -1) {
                 glsafe(::glEnableVertexAttribArray(position_id));
@@ -484,7 +470,8 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
                 glsafe(::glFrontFace(GL_CCW));
 
             glsafe(::glDisable(GL_BLEND));
-            glsafe(::glDepthMask(GL_TRUE));
+            if (bottom)
+                glsafe(::glDepthMask(GL_TRUE));
 
             shader->stop_using();
         }
@@ -496,34 +483,36 @@ void Bed3D::render_model() const
     if (m_model_filename.empty())
         return;
 
-    if ((m_model.get_filename() != m_model_filename) && m_model.init_from_file(m_model_filename)) {
+    GLModel* model = const_cast<GLModel*>(&m_model);
+
+    if (model->get_filename() != m_model_filename && model->init_from_file(m_model_filename)) {
+        model->set_color(-1, DEFAULT_MODEL_COLOR);
+
         // move the model so that its origin (0.0, 0.0, 0.0) goes into the bed shape center and a bit down to avoid z-fighting with the texture quad
-        Vec3d shift = m_bounding_box.center();
-        shift(2) = -0.03;
-        m_model_offset = shift;
+        *const_cast<Vec3d*>(&m_model_offset) = to_3d(m_build_volume.bounding_volume2d().center(), -0.03);
 
         // update extended bounding box
-        calc_bounding_boxes();
+        const_cast<BoundingBoxf3&>(m_extended_bounding_box) = this->calc_extended_bounding_box();
     }
 
-    if (!m_model.get_filename().empty()) {
+    if (!model->get_filename().empty()) {
         GLShaderProgram* shader = wxGetApp().get_shader("gouraud_light");
         if (shader != nullptr) {
             shader->start_using();
-            shader->set_uniform("uniform_color", m_model_color);
-            ::glPushMatrix();
-            ::glTranslated(m_model_offset(0), m_model_offset(1), m_model_offset(2));
-            m_model.render();
-            ::glPopMatrix();
+            shader->set_uniform("emission_factor", 0.0f);
+            glsafe(::glPushMatrix());
+            glsafe(::glTranslated(m_model_offset.x(), m_model_offset.y(), m_model_offset.z()));
+            model->render();
+            glsafe(::glPopMatrix());
             shader->stop_using();
         }
     }
 }
 
-void Bed3D::render_custom(GLCanvas3D& canvas, bool bottom, bool show_texture) const
+void Bed3D::render_custom(GLCanvas3D& canvas, bool bottom, bool show_texture, bool picking) const
 {
     if (m_texture_filename.empty() && m_model_filename.empty()) {
-        render_default(bottom);
+        render_default(bottom, picking);
         return;
     }
 
@@ -534,9 +523,9 @@ void Bed3D::render_custom(GLCanvas3D& canvas, bool bottom, bool show_texture) co
         render_texture(bottom, canvas);
 }
 
-void Bed3D::render_default(bool bottom) const
+void Bed3D::render_default(bool bottom, bool picking) const
 {
-    m_texture.reset();
+    const_cast<GLTexture*>(&m_texture)->reset();
 
     unsigned int triangles_vcount = m_triangles.get_vertices_count();
     if (triangles_vcount > 0) {
@@ -551,21 +540,23 @@ void Bed3D::render_default(bool bottom) const
         if (!has_model && !bottom) {
             // draw background
             glsafe(::glDepthMask(GL_FALSE));
-            glsafe(::glColor4fv(m_model_color.data()));
+            glsafe(::glColor4fv(picking ? PICKING_MODEL_COLOR.data() : DEFAULT_MODEL_COLOR.data()));
             glsafe(::glNormal3d(0.0f, 0.0f, 1.0f));
             glsafe(::glVertexPointer(3, GL_FLOAT, m_triangles.get_vertex_data_size(), (GLvoid*)m_triangles.get_vertices_data()));
             glsafe(::glDrawArrays(GL_TRIANGLES, 0, (GLsizei)triangles_vcount));
             glsafe(::glDepthMask(GL_TRUE));
         }
 
-        // draw grid
-        glsafe(::glLineWidth(1.5f * m_scale_factor));
-        if (has_model && !bottom)
-            glsafe(::glColor4f(0.9f, 0.9f, 0.9f, 1.0f));
-        else
-            glsafe(::glColor4f(0.9f, 0.9f, 0.9f, 0.6f));
-        glsafe(::glVertexPointer(3, GL_FLOAT, m_triangles.get_vertex_data_size(), (GLvoid*)m_gridlines.get_vertices_data()));
-        glsafe(::glDrawArrays(GL_LINES, 0, (GLsizei)m_gridlines.get_vertices_count()));
+        if (!picking) {
+            // draw grid
+            glsafe(::glLineWidth(1.5f * m_scale_factor));
+            if (has_model && !bottom)
+                glsafe(::glColor4f(0.9f, 0.9f, 0.9f, 1.0f));
+            else
+                glsafe(::glColor4f(0.9f, 0.9f, 0.9f, 0.6f));
+            glsafe(::glVertexPointer(3, GL_FLOAT, m_triangles.get_vertex_data_size(), (GLvoid*)m_gridlines.get_vertices_data()));
+            glsafe(::glDrawArrays(GL_LINES, 0, (GLsizei)m_gridlines.get_vertices_count()));
+        }
 
         glsafe(::glDisableClientState(GL_VERTEX_ARRAY));
 
@@ -573,7 +564,7 @@ void Bed3D::render_default(bool bottom) const
     }
 }
 
-void Bed3D::reset()
+void Bed3D::release_VBOs()
 {
     if (m_vbo_id > 0) {
         glsafe(::glDeleteBuffers(1, &m_vbo_id));
