@@ -8,11 +8,14 @@
 #include "libslic3r/Utils.hpp"
 #include "libslic3r/Geometry.hpp"
 
-#include <functional>
+#include "GLModel.hpp"
 
-#if ENABLE_OPENGL_ERROR_LOGGING || ! defined(NDEBUG)
-    #define HAS_GLSAFE
-#endif
+#include <functional>
+#include <optional>
+
+#ifndef NDEBUG
+#define HAS_GLSAFE
+#endif // NDEBUG
 
 #ifdef HAS_GLSAFE
     extern void glAssertRecentCallImpl(const char *file_name, unsigned int line, const char *function_name);
@@ -28,6 +31,7 @@
 namespace Slic3r {
 class SLAPrintObject;
 enum  SLAPrintObjectStep : unsigned int;
+class BuildVolume;
 class DynamicPrintConfig;
 class ExtrusionPath;
 class ExtrusionMultiPath;
@@ -38,30 +42,29 @@ class ModelObject;
 class ModelVolume;
 enum ModelInstanceEPrintVolumeState : unsigned char;
 
+// Return appropriate color based on the ModelVolume.
+std::array<float, 4> color_from_model_volume(const ModelVolume& model_volume);
+
 // A container for interleaved arrays of 3D vertices and normals,
 // possibly indexed by triangles and / or quads.
 class GLIndexedVertexArray {
 public:
-    GLIndexedVertexArray() : 
-        vertices_and_normals_interleaved_VBO_id(0),
-        triangle_indices_VBO_id(0),
-        quad_indices_VBO_id(0)
-        {}
+    // Only Eigen types of Nx16 size are vectorized. This bounding box will not be vectorized.
+    static_assert(sizeof(Eigen::AlignedBox<float, 3>) == 24, "Eigen::AlignedBox<float, 3> is not being vectorized, thus it does not need to be aligned");
+    using BoundingBox = Eigen::AlignedBox<float, 3>;
+
+    GLIndexedVertexArray() { m_bounding_box.setEmpty(); }
     GLIndexedVertexArray(const GLIndexedVertexArray &rhs) :
         vertices_and_normals_interleaved(rhs.vertices_and_normals_interleaved),
         triangle_indices(rhs.triangle_indices),
         quad_indices(rhs.quad_indices),
-        vertices_and_normals_interleaved_VBO_id(0),
-        triangle_indices_VBO_id(0),
-        quad_indices_VBO_id(0)
-        { assert(! rhs.has_VBOs()); }
+        m_bounding_box(rhs.m_bounding_box)
+        { assert(! rhs.has_VBOs()); m_bounding_box.setEmpty(); }
     GLIndexedVertexArray(GLIndexedVertexArray &&rhs) :
         vertices_and_normals_interleaved(std::move(rhs.vertices_and_normals_interleaved)),
         triangle_indices(std::move(rhs.triangle_indices)),
         quad_indices(std::move(rhs.quad_indices)),
-        vertices_and_normals_interleaved_VBO_id(0),
-        triangle_indices_VBO_id(0),
-        quad_indices_VBO_id(0)
+        m_bounding_box(rhs.m_bounding_box)
         { assert(! rhs.has_VBOs()); }
 
     ~GLIndexedVertexArray() { release_geometry(); }
@@ -95,7 +98,7 @@ public:
         this->vertices_and_normals_interleaved 		 = std::move(rhs.vertices_and_normals_interleaved);
         this->triangle_indices                 		 = std::move(rhs.triangle_indices);
         this->quad_indices                     		 = std::move(rhs.quad_indices);
-        this->m_bounding_box                   		 = std::move(rhs.m_bounding_box);
+        this->m_bounding_box                   		 = rhs.m_bounding_box;
         this->vertices_and_normals_interleaved_size  = rhs.vertices_and_normals_interleaved_size;
         this->triangle_indices_size                  = rhs.triangle_indices_size;
         this->quad_indices_size                      = rhs.quad_indices_size;
@@ -127,6 +130,8 @@ public:
     void load_mesh(const TriangleMesh& mesh) { this->load_mesh_full_shading(mesh); }
 #endif // ENABLE_SMOOTH_NORMALS
 
+    void load_its_flat_shading(const indexed_triangle_set &its);
+
     inline bool has_VBOs() const { return vertices_and_normals_interleaved_VBO_id != 0; }
 
     inline void reserve(size_t sz) {
@@ -150,15 +155,16 @@ public:
         this->vertices_and_normals_interleaved.emplace_back(z);
 
         this->vertices_and_normals_interleaved_size = this->vertices_and_normals_interleaved.size();
-        m_bounding_box.merge(Vec3f(x, y, z).cast<double>());
+        m_bounding_box.extend(Vec3f(x, y, z));
     };
 
     inline void push_geometry(double x, double y, double z, double nx, double ny, double nz) {
         push_geometry(float(x), float(y), float(z), float(nx), float(ny), float(nz));
     }
 
-    inline void push_geometry(const Vec3d& p, const Vec3d& n) {
-        push_geometry(p(0), p(1), p(2), n(0), n(1), n(2));
+    template<typename Derived, typename Derived2>
+    inline void push_geometry(const Eigen::MatrixBase<Derived>& p, const Eigen::MatrixBase<Derived2>& n) {
+        push_geometry(float(p(0)), float(p(1)), float(p(2)), float(n(0)), float(n(1)), float(n(2)));
     }
 
     inline void push_triangle(int idx1, int idx2, int idx3) {
@@ -205,10 +211,10 @@ public:
         this->vertices_and_normals_interleaved.clear();
         this->triangle_indices.clear();
         this->quad_indices.clear();
-        this->m_bounding_box.reset();
         vertices_and_normals_interleaved_size = 0;
         triangle_indices_size = 0;
         quad_indices_size = 0;
+        m_bounding_box.setEmpty();
     }
 
     // Shrink the internal storage to tighly fit the data stored.
@@ -218,7 +224,7 @@ public:
         this->quad_indices.shrink_to_fit();
     }
 
-    const BoundingBoxf3& bounding_box() const { return m_bounding_box; }
+    const BoundingBox& bounding_box() const { return m_bounding_box; }
 
     // Return an estimate of the memory consumed by this class.
     size_t cpu_memory_used() const { return sizeof(*this) + vertices_and_normals_interleaved.capacity() * sizeof(float) + triangle_indices.capacity() * sizeof(int) + quad_indices.capacity() * sizeof(int); }
@@ -237,54 +243,72 @@ public:
     size_t total_memory_used() const { return this->cpu_memory_used() + this->gpu_memory_used(); }
 
 private:
-    BoundingBoxf3 m_bounding_box;
+    BoundingBox m_bounding_box;
 };
 
 class GLVolume {
 public:
-    static const float SELECTED_COLOR[4];
-    static const float HOVER_SELECT_COLOR[4];
-    static const float HOVER_DESELECT_COLOR[4];
-    static const float OUTSIDE_COLOR[4];
-    static const float SELECTED_OUTSIDE_COLOR[4];
-    static const float DISABLED_COLOR[4];
-    static const float MODEL_COLOR[4][4];
-    static const float SLA_SUPPORT_COLOR[4];
-    static const float SLA_PAD_COLOR[4];
-    static const float NEUTRAL_COLOR[4];
+    static const std::array<float, 4> SELECTED_COLOR;
+    static const std::array<float, 4> HOVER_SELECT_COLOR;
+    static const std::array<float, 4> HOVER_DESELECT_COLOR;
+    static const std::array<float, 4> OUTSIDE_COLOR;
+    static const std::array<float, 4> SELECTED_OUTSIDE_COLOR;
+    static const std::array<float, 4> DISABLED_COLOR;
+    static const std::array<float, 4> SLA_SUPPORT_COLOR;
+    static const std::array<float, 4> SLA_PAD_COLOR;
+    static const std::array<float, 4> NEUTRAL_COLOR;
+    static const std::array<std::array<float, 4>, 4> MODEL_COLOR;
 
     enum EHoverState : unsigned char
     {
         HS_None,
+        HS_Hover,
         HS_Select,
         HS_Deselect
     };
 
     GLVolume(float r = 1.f, float g = 1.f, float b = 1.f, float a = 1.f);
-    GLVolume(const float *rgba) : GLVolume(rgba[0], rgba[1], rgba[2], rgba[3]) {}
+    GLVolume(const std::array<float, 4>& rgba) : GLVolume(rgba[0], rgba[1], rgba[2], rgba[3]) {}
 
 private:
     Geometry::Transformation m_instance_transformation;
     Geometry::Transformation m_volume_transformation;
 
     // Shift in z required by sla supports+pad
-    double        		  m_sla_shift_z;
+    double        m_sla_shift_z;
     // Bounding box of this volume, in unscaled coordinates.
-    mutable BoundingBoxf3 m_transformed_bounding_box;
-    // Whether or not is needed to recalculate the transformed bounding box.
-    mutable bool          m_transformed_bounding_box_dirty;
+    std::optional<BoundingBoxf3> m_transformed_bounding_box;
     // Convex hull of the volume, if any.
     std::shared_ptr<const TriangleMesh> m_convex_hull;
     // Bounding box of this volume, in unscaled coordinates.
-    mutable BoundingBoxf3 m_transformed_convex_hull_bounding_box;
-    // Whether or not is needed to recalculate the transformed convex hull bounding box.
-    mutable bool          m_transformed_convex_hull_bounding_box_dirty;
+    std::optional<BoundingBoxf3> m_transformed_convex_hull_bounding_box;
+    // Bounding box of the non sinking part of this volume, in unscaled coordinates.
+    std::optional<BoundingBoxf3> m_transformed_non_sinking_bounding_box;
+
+    class SinkingContours
+    {
+        static const float HalfWidth;
+        GLVolume& m_parent;
+        GUI::GLModel m_model;
+        BoundingBoxf3 m_old_box;
+        Vec3d m_shift{ Vec3d::Zero() };
+
+    public:
+        SinkingContours(GLVolume& volume) : m_parent(volume) {}
+        void render();
+
+    private:
+        void update();
+    };
+
+    SinkingContours m_sinking_contours;
 
 public:
     // Color of the triangles / quads held by this volume.
-    float               color[4];
+    std::array<float, 4> color;
     // Color used to render this volume.
-    float               render_color[4];
+    std::array<float, 4> render_color;
+
     struct CompositeID {
         CompositeID(int object_id, int volume_id, int instance_id) : object_id(object_id), volume_id(volume_id), instance_id(instance_id) {}
         CompositeID() : object_id(-1), volume_id(-1), instance_id(-1) {}
@@ -339,7 +363,9 @@ public:
 	    bool                force_native_color : 1;
         // Whether or not render this volume in neutral
         bool                force_neutral_color : 1;
-	};
+        // Whether or not to force rendering of sinking contours
+        bool                force_sinking_contours : 1;
+    };
 
     // Is mouse or rectangle selection over this object to select/deselect it ?
     EHoverState         	hover;
@@ -357,14 +383,23 @@ public:
     std::vector<size_t>         offsets;
 
     // Bounding box of this volume, in unscaled coordinates.
-    const BoundingBoxf3& bounding_box() const { return this->indexed_vertex_array.bounding_box(); }
+    BoundingBoxf3 bounding_box() const { 
+        BoundingBoxf3 out;
+        if (! this->indexed_vertex_array.bounding_box().isEmpty()) {
+            out.min = this->indexed_vertex_array.bounding_box().min().cast<double>();
+            out.max = this->indexed_vertex_array.bounding_box().max().cast<double>();
+            out.defined = true;
+        };
+        return out;
+    }
 
+    void set_color(const std::array<float, 4>& rgba);
     void set_render_color(float r, float g, float b, float a);
-    void set_render_color(const float* rgba, unsigned int size);
+    void set_render_color(const std::array<float, 4>& rgba);
     // Sets render color in dependence of current state
     void set_render_color();
     // set color according to model volume
-    void set_color_from_model_volume(const ModelVolume *model_volume);
+    void set_color_from_model_volume(const ModelVolume& model_volume);
 
     const Geometry::Transformation& get_instance_transformation() const { return m_instance_transformation; }
     void set_instance_transformation(const Geometry::Transformation& transformation) { m_instance_transformation = transformation; set_bounding_boxes_as_dirty(); }
@@ -439,6 +474,10 @@ public:
     BoundingBoxf3        transformed_convex_hull_bounding_box(const Transform3d &trafo) const;
     // caching variant
     const BoundingBoxf3& transformed_convex_hull_bounding_box() const;
+    // non-caching variant
+    BoundingBoxf3        transformed_non_sinking_bounding_box(const Transform3d& trafo) const;
+    // caching variant
+    const BoundingBoxf3& transformed_non_sinking_bounding_box() const;
     // convex hull
     const TriangleMesh*  convex_hull() const { return m_convex_hull.get(); }
 
@@ -451,11 +490,19 @@ public:
     void                finalize_geometry(bool opengl_initialized) { this->indexed_vertex_array.finalize_geometry(opengl_initialized); }
     void                release_geometry() { this->indexed_vertex_array.release_geometry(); }
 
-    void                set_bounding_boxes_as_dirty() { m_transformed_bounding_box_dirty = true; m_transformed_convex_hull_bounding_box_dirty = true; }
+    void                set_bounding_boxes_as_dirty() {
+        m_transformed_bounding_box.reset();
+        m_transformed_convex_hull_bounding_box.reset();
+        m_transformed_non_sinking_bounding_box.reset();
+    }
 
     bool                is_sla_support() const;
     bool                is_sla_pad() const;
 
+    bool                is_sinking() const;
+    bool                is_below_printbed() const;
+    void                render_sinking_contours();
+
     // Return an estimate of the memory consumed by this class.
     size_t 				cpu_memory_used() const { 
     	//FIXME what to do wih m_convex_hull?
@@ -473,17 +520,29 @@ typedef std::vector<GLVolumeWithIdAndZ> GLVolumeWithIdAndZList;
 class GLVolumeCollection
 {
 public:
-    enum ERenderType : unsigned char
+    enum class ERenderType : unsigned char
     {
         Opaque,
         Transparent,
         All
     };
 
+    struct PrintVolume
+    {
+        // see: Bed3D::EShapeType
+        int type{ 0 };
+        // data contains:
+        // Rectangle:
+        //   [0] = min.x, [1] = min.y, [2] = max.x, [3] = max.y
+        // Circle:
+        //   [0] = center.x, [1] = center.y, [3] = radius
+        std::array<float, 4> data;
+        //   [0] = min z, [1] = max z
+        std::array<float, 2> zs;
+    };
+
 private:
-    // min and max vertex of the print box volume
-    float m_print_box_min[3];
-    float m_print_box_max[3];
+    PrintVolume m_print_volume;
 
     // z range for clipping in shaders
     float m_z_range[2];
@@ -499,6 +558,7 @@ private:
     };
 
     Slope m_slope;
+    bool m_show_sinking_contours = false;
 
 public:
     GLVolumePtrs volumes;
@@ -535,8 +595,8 @@ public:
     int load_wipe_tower_preview(
         int obj_idx, float pos_x, float pos_y, float width, float depth, float height, float rotation_angle, bool size_unknown, float brim_width, bool opengl_initialized);
 
-    GLVolume* new_toolpath_volume(const float *rgba, size_t reserve_vbo_floats = 0);
-    GLVolume* new_nontoolpath_volume(const float *rgba, size_t reserve_vbo_floats = 0);
+    GLVolume* new_toolpath_volume(const std::array<float, 4>& rgba, size_t reserve_vbo_floats = 0);
+    GLVolume* new_nontoolpath_volume(const std::array<float, 4>& rgba, size_t reserve_vbo_floats = 0);
 
     // Render the volumes by OpenGL.
     void render(ERenderType type, bool disable_cullface, const Transform3d& view_matrix, std::function<bool(const GLVolume&)> filter_func = std::function<bool(const GLVolume&)>()) const;
@@ -554,10 +614,7 @@ public:
     bool empty() const { return volumes.empty(); }
     void set_range(double low, double high) { for (GLVolume *vol : this->volumes) vol->set_range(low, high); }
 
-    void set_print_box(float min_x, float min_y, float min_z, float max_x, float max_y, float max_z) {
-        m_print_box_min[0] = min_x; m_print_box_min[1] = min_y; m_print_box_min[2] = min_z;
-        m_print_box_max[0] = max_x; m_print_box_max[1] = max_y; m_print_box_max[2] = max_z;
-    }
+    void set_print_volume(const PrintVolume& print_volume) { m_print_volume = print_volume; }
 
     void set_z_range(float min_z, float max_z) { m_z_range[0] = min_z; m_z_range[1] = max_z; }
     void set_clipping_plane(const double* coeffs) { m_clipping_plane[0] = coeffs[0]; m_clipping_plane[1] = coeffs[1]; m_clipping_plane[2] = coeffs[2]; m_clipping_plane[3] = coeffs[3]; }
@@ -568,10 +625,11 @@ public:
     float get_slope_normal_z() const { return m_slope.normal_z; }
     void set_slope_normal_z(float normal_z) { m_slope.normal_z = normal_z; }
     void set_default_slope_normal_z() { m_slope.normal_z = -::cos(Geometry::deg2rad(90.0f - 45.0f)); }
+    void set_show_sinking_contours(bool show) { m_show_sinking_contours = show; }
 
     // returns true if all the volumes are completely contained in the print volume
     // returns the containment state in the given out_state, if non-null
-    bool check_outside_state(const DynamicPrintConfig* config, ModelInstanceEPrintVolumeState* out_state);
+    bool check_outside_state(const Slic3r::BuildVolume& build_volume, ModelInstanceEPrintVolumeState* out_state) const;
     void reset_outside_state();
 
     void update_colors_by_extruder(const DynamicPrintConfig* config);
@@ -587,8 +645,6 @@ public:
     // Return CPU, GPU and total memory log line.
     std::string         log_memory_info() const;
 
-    bool                has_toolpaths_to_export() const;
-
 private:
     GLVolumeCollection(const GLVolumeCollection &other);
     GLVolumeCollection& operator=(const GLVolumeCollection &);
@@ -611,8 +667,6 @@ struct _3DScene
     static void point3_to_verts(const Vec3crd& point, double width, double height, GLVolume& volume);
 };
 
-static constexpr float BedEpsilon = 3.f * float(EPSILON);
-
 }
 
 #endif
