@@ -21,21 +21,27 @@ class ConfigManipulation
 {
     bool                is_msg_dlg_already_exist{ false };
 
+    bool                m_is_initialized_support_material_overhangs_queried{ false };
+    bool                m_support_material_overhangs_queried{ false };
+
     // function to loading of changed configuration 
     std::function<void()>                                       load_config = nullptr;
     std::function<void (const std::string&, bool toggle, int opt_index)>   cb_toggle_field = nullptr;
     // callback to propagation of changed value, if needed 
     std::function<void(const std::string&, const boost::any&)>  cb_value_change = nullptr;
     ModelConfig* local_config = nullptr;
+    wxWindow*    m_msg_dlg_parent {nullptr};
 
 public:
     ConfigManipulation(std::function<void()> load_config,
         std::function<void(const std::string&, bool toggle, int opt_index)> cb_toggle_field,
         std::function<void(const std::string&, const boost::any&)>  cb_value_change,
-        ModelConfig* local_config = nullptr) :
+        ModelConfig* local_config = nullptr,
+        wxWindow* msg_dlg_parent  = nullptr) :
         load_config(load_config),
         cb_toggle_field(cb_toggle_field),
         cb_value_change(cb_value_change),
+        m_msg_dlg_parent(msg_dlg_parent),
         local_config(local_config) {}
     ConfigManipulation() {}
 
@@ -55,6 +61,13 @@ public:
     // SLA print
     void    update_print_sla_config(DynamicPrintConfig* config, const bool is_global_config = false);
     void    toggle_print_sla_options(DynamicPrintConfig* config);
+
+    bool    is_initialized_support_material_overhangs_queried() { return m_is_initialized_support_material_overhangs_queried; }
+    void    initialize_support_material_overhangs_queried(bool queried)
+    {
+        m_is_initialized_support_material_overhangs_queried = true;
+        m_support_material_overhangs_queried = queried;
+    }
 };
 
 } // GUI
