@@ -877,10 +877,10 @@ void SpinCtrl::BUILD() {
 
 void SpinCtrl::propagate_value()
 {
-    if (suppress_propagation)
+    // check if value was really changed
+    if (boost::any_cast<int>(m_value) == tmp_value)
         return;
 
-    suppress_propagation = true;
     if (tmp_value == UNDEF_VALUE) {
         on_kill_focus();
 	} else {
@@ -894,7 +894,6 @@ void SpinCtrl::propagate_value()
 #endif
         on_change_field();
     }
-    suppress_propagation = false;
 }
 
 void SpinCtrl::msw_rescale()
@@ -1048,8 +1047,7 @@ void Choice::set_selection()
     choice_ctrl* field = dynamic_cast<choice_ctrl*>(window);
 	switch (m_opt.type) {
 	case coEnum:{
-		int id_value = m_opt.get_default_value<ConfigOptionEnum<SeamPosition>>()->value; //!!
-        field->SetSelection(id_value);
+        field->SetSelection(m_opt.default_value->getInt());
 		break;
 	}
 	case coFloat:
