@@ -448,9 +448,8 @@ class GCodeViewer
             size_t first{ 0 };
             size_t last{ 0 };
 
-            bool operator == (const Endpoints& other) const {
-                return first == other.first && last == other.last;
-            }
+            bool operator == (const Endpoints& other) const { return first == other.first && last == other.last; }
+            bool operator != (const Endpoints& other) const { return !operator==(other); }
         };
 
     private:
@@ -479,9 +478,8 @@ class GCodeViewer
         bool operator != (const Layers& other) const {
             if (m_zs != other.m_zs)
                 return true;
-            if (!(m_endpoints == other.m_endpoints))
+            if (m_endpoints != other.m_endpoints)
                 return true;
-
             return false;
         }
     };
