@@ -1,9 +1,12 @@
 #include "GUI_ObjectSettings.hpp"
 #include "GUI_ObjectList.hpp"
+#include "GUI_Factories.hpp"
 
 #include "OptionsGroup.hpp"
+#include "GUI_App.hpp"
 #include "wxExtensions.hpp"
-#include "PresetBundle.hpp"
+#include "Plater.hpp"
+#include "libslic3r/PresetBundle.hpp"
 #include "libslic3r/Model.hpp"
 
 #include <boost/algorithm/string.hpp>
@@ -56,6 +59,7 @@ wxSizer* OG_Settings::get_sizer()
 ObjectSettings::ObjectSettings(wxWindow* parent) :
     OG_Settings(parent, true)
 {
+    m_og->activate();
     m_og->set_name(_(L("Additional Settings")));    
 
     m_settings_list_sizer = new wxBoxSizer(wxVERTICAL);
@@ -80,7 +84,7 @@ bool ObjectSettings::update_settings_list()
         return false;
 
     const bool is_object_settings = objects_model->GetItemType(objects_model->GetParent(item)) == itObject;
-	SettingsBundle cat_options = objects_ctrl->get_item_settings_bundle(config, is_object_settings);
+    SettingsFactory::Bundle cat_options = SettingsFactory::get_bundle(&config->get(), is_object_settings);
 
     if (!cat_options.empty())
     {
@@ -106,12 +110,6 @@ bool ObjectSettings::update_settings_list()
                     update_settings_list(); 
                     m_parent->Layout(); 
                 });
-
-                /* Check overriden options list after deleting.
-                 * Some options couldn't be deleted because of another one.
-                 * Like, we couldn't delete fill pattern, if fill density is set to 100%
-                 */
-                update_config_values(config);
 			});
 			return btn;
 		};
@@ -143,17 +141,20 @@ bool ObjectSettings::update_settings_list()
             {
                 Option option = optgroup->get_option(opt);
                 option.opt.width = 12;
+                if (!option.opt.full_label.empty())
+                    option.opt.label = option.opt.full_label;
                 if (is_extruders_cat)
                     option.opt.max = wxGetApp().extruders_edited_cnt();
                 optgroup->append_single_option_line(option);
-
+            }
+            optgroup->activate();
+            for (auto& opt : cat.second)
                 optgroup->get_field(opt)->m_on_change = [optgroup](const std::string& opt_id, const boost::any& value) {
                     // first of all take a snapshot and then change value in configuration
                     wxGetApp().plater()->take_snapshot(from_u8((boost::format(_utf8(L("Change Option %s"))) % opt_id).str()));
                     optgroup->on_change_OG(opt_id, value);
                 };
 
-            }
             optgroup->reload_config();
 
             m_settings_list_sizer->Add(optgroup->sizer, 0, wxEXPAND | wxALL, 0);
@@ -174,24 +175,23 @@ bool ObjectSettings::update_settings_list()
     return true;
 }
 
-bool ObjectSettings::add_missed_options(DynamicPrintConfig* config_to, const DynamicPrintConfig& config_from)
+bool ObjectSettings::add_missed_options(ModelConfig* config_to, const DynamicPrintConfig& config_from)
 {
+    const DynamicPrintConfig& print_config = wxGetApp().plater()->printer_technology() == ptFFF ?
+                                             wxGetApp().preset_bundle->prints.get_edited_preset().config :
+                                             wxGetApp().preset_bundle->sla_prints.get_edited_preset().config;
     bool is_added = false;
-    if (wxGetApp().plater()->printer_technology() == ptFFF)
-    {
-        if (config_to->has("fill_density") && !config_to->has("fill_pattern"))
-        {
-            if (config_from.option<ConfigOptionPercent>("fill_density")->value == 100) {
-                config_to->set_key_value("fill_pattern", config_from.option("fill_pattern")->clone());
-                is_added = true;
-            }
+
+    for (auto opt_key : config_from.diff(print_config))
+        if (!config_to->has(opt_key)) {
+            config_to->set_key_value(opt_key, config_from.option(opt_key)->clone());
+            is_added = true;
         }
-    }
 
     return is_added;
 }
 
-void ObjectSettings::update_config_values(DynamicPrintConfig* config)
+void ObjectSettings::update_config_values(ModelConfig* config)
 {
     const auto objects_model        = wxGetApp().obj_list()->GetModel();
     const auto item                 = wxGetApp().obj_list()->GetSelection();
@@ -223,39 +223,39 @@ void ObjectSettings::update_config_values(DynamicPrintConfig* config)
         update_config_values(config);
 
         if (is_added) {
-            wxTheApp->CallAfter([this]() {
+// #ysFIXME - Delete after testing! Very likely this CallAfret is no needed
+//            wxTheApp->CallAfter([this]() {
                 wxWindowUpdateLocker noUpdates(m_parent);
                 update_settings_list();
                 m_parent->Layout();
-            });
+//            });
         }
     };
 
-    auto get_field = [this](const t_config_option_key & opt_key, int opt_index)
+    auto toggle_field = [this](const t_config_option_key & opt_key, bool toggle, int opt_index)
     {
         Field* field = nullptr;
         for (auto og : m_og_settings) {
             field = og->get_fieldc(opt_key, opt_index);
             if (field != nullptr)
-                return field;
+                break;
         }
-        return field;
+        if (field)
+            field->toggle(toggle);
     };
 
-    ConfigManipulation config_manipulation(load_config, get_field, nullptr, config);
+    ConfigManipulation config_manipulation(load_config, toggle_field, nullptr, config);
 
     if (!is_object_settings)
     {
         const int obj_idx = objects_model->GetObjectIdByItem(item);
         assert(obj_idx >= 0);
-        DynamicPrintConfig* obj_config = &wxGetApp().model().objects[obj_idx]->config;
-
-        main_config.apply(*obj_config, true);
-        printer_technology == ptFFF  ?  config_manipulation.update_print_fff_config(&main_config) :
-                                        config_manipulation.update_print_sla_config(&main_config) ;
+        // for object's part first of all update konfiguration from object 
+        main_config.apply(wxGetApp().model().objects[obj_idx]->config.get(), true);
+        // and then from its own config
     }
 
-    main_config.apply(*config, true);
+    main_config.apply(config->get(), true);
     printer_technology == ptFFF  ?  config_manipulation.update_print_fff_config(&main_config) :
                                     config_manipulation.update_print_sla_config(&main_config) ;
 
@@ -277,5 +277,13 @@ void ObjectSettings::msw_rescale()
         group->msw_rescale();
 }
 
+void ObjectSettings::sys_color_changed()
+{
+    m_og->sys_color_changed();
+
+    for (auto group : m_og_settings)
+        group->sys_color_changed();
+}
+
 } //namespace GUI
 } //namespace Slic3r 
