@@ -9,13 +9,11 @@
 #include <string>
 #include "libslic3r/GCode/GCodeProcessor.hpp"
 
-class wxNotebook;
 class wxGLCanvas;
 class wxBoxSizer;
 class wxStaticText;
-class wxChoice;
+class wxComboBox;
 class wxComboCtrl;
-class wxBitmapComboBox;
 class wxCheckBox;
 
 namespace Slic3r {
@@ -36,6 +34,9 @@ class GLToolbar;
 class Bed3D;
 struct Camera;
 class Plater;
+#ifdef _WIN32
+class BitmapComboBox;
+#endif
 
 class View3D : public wxPanel
 {
@@ -43,7 +44,7 @@ class View3D : public wxPanel
     GLCanvas3D* m_canvas;
 
 public:
-    View3D(wxWindow* parent, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process);
+    View3D(wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process);
     virtual ~View3D();
 
     wxGLCanvas* get_wxglcanvas() { return m_canvas_widget; }
@@ -58,8 +59,6 @@ public:
     void delete_selected();
     void mirror_selection(Axis axis);
 
-    int check_volumes_outside_state() const;
-
     bool is_layers_editing_enabled() const;
     bool is_layers_editing_allowed() const;
     void enable_layers_editing(bool enable);
@@ -71,7 +70,7 @@ public:
     void render();
 
 private:
-    bool init(wxWindow* parent, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process);
+    bool init(wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process);
 };
 
 class Preview : public wxPanel
@@ -82,7 +81,11 @@ class Preview : public wxPanel
     wxBoxSizer* m_layers_slider_sizer { nullptr };
     wxPanel* m_bottom_toolbar_panel { nullptr };
     wxStaticText* m_label_view_type { nullptr };
-    wxChoice* m_choice_view_type { nullptr };
+#ifdef _WIN32
+    BitmapComboBox* m_choice_view_type { nullptr };
+#else
+    wxComboBox* m_choice_view_type { nullptr };
+#endif
     wxStaticText* m_label_show { nullptr };
     wxComboCtrl* m_combochecklist_features { nullptr };
     size_t m_combochecklist_features_pos { 0 };
@@ -90,7 +93,7 @@ class Preview : public wxPanel
 
     DynamicPrintConfig* m_config;
     BackgroundSlicingProcess* m_process;
-    GCodeProcessor::Result* m_gcode_result;
+    GCodeProcessorResult* m_gcode_result;
 
 #ifdef __linux__
     // We are getting mysterious crashes on Linux in gtk due to OpenGL context activation GH #1874 #1955.
@@ -102,11 +105,7 @@ class Preview : public wxPanel
     std::function<void()> m_schedule_background_process;
 
     unsigned int m_number_extruders { 1 };
-#if ENABLE_PREVIEW_TYPE_CHANGE
     bool m_keep_current_preview_type{ false };
-#else
-    std::string m_preferred_color_mode;
-#endif // ENABLE_PREVIEW_TYPE_CHANGE
 
     bool m_loaded { false };
 
@@ -120,6 +119,7 @@ public:
         Wipe,
         Retractions,
         Unretractions,
+        Seams,
         ToolChanges,
         ColorChanges,
         PausePrints,
@@ -129,8 +129,8 @@ public:
         Legend
     };
 
-    Preview(wxWindow* parent, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process, 
-        GCodeProcessor::Result* gcode_result, std::function<void()> schedule_background_process = []() {});
+    Preview(wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process, 
+        GCodeProcessorResult* gcode_result, std::function<void()> schedule_background_process = []() {});
     virtual ~Preview();
 
     wxGLCanvas* get_wxglcanvas() { return m_canvas_widget; }
@@ -138,9 +138,6 @@ public:
 
     void set_as_dirty();
 
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-    void set_number_extruders(unsigned int number_extruders);
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
     void bed_shape_changed();
     void select_view(const std::string& direction);
     void set_drop_target(wxDropTarget* target);
@@ -155,22 +152,16 @@ public:
     void move_layers_slider(wxKeyEvent& evt);
     void edit_layers_slider(wxKeyEvent& evt);
 
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-    void update_view_type(bool keep_volumes);
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
-
     bool is_loaded() const { return m_loaded; }
 
     void update_bottom_toolbar();
     void update_moves_slider();
     void enable_moves_slider(bool enable);
-#if ENABLE_ARROW_KEYS_WITH_SLIDERS
     void move_moves_slider(wxKeyEvent& evt);
-#endif // ENABLE_ARROW_KEYS_WITH_SLIDERS
     void hide_layers_slider();
 
 private:
-    bool init(wxWindow* parent, Model* model);
+    bool init(wxWindow* parent, Bed3D& bed, Model* model);
 
     void bind_event_handlers();
     void unbind_event_handlers();
