@@ -1,6 +1,7 @@
 // Include GLGizmoBase.hpp before I18N.hpp as it includes some libigl code, which overrides our localization "L" macro.
 #include "GLGizmoMove.hpp"
 #include "slic3r/GUI/GLCanvas3D.hpp"
+#include "slic3r/GUI/GUI_App.hpp"
 
 #include <GL/glew.h>
 
@@ -18,17 +19,8 @@ GLGizmoMove3D::GLGizmoMove3D(GLCanvas3D& parent, const std::string& icon_filenam
     , m_starting_drag_position(Vec3d::Zero())
     , m_starting_box_center(Vec3d::Zero())
     , m_starting_box_bottom_center(Vec3d::Zero())
-    , m_quadric(nullptr)
 {
-    m_quadric = ::gluNewQuadric();
-    if (m_quadric != nullptr)
-        ::gluQuadricDrawStyle(m_quadric, GLU_FILL);
-}
-
-GLGizmoMove3D::~GLGizmoMove3D()
-{
-    if (m_quadric != nullptr)
-        ::gluDeleteQuadric(m_quadric);
+    m_vbo_cone.init_from(its_make_cone(1., 1., 2*PI/36));
 }
 
 std::string GLGizmoMove3D::get_tooltip() const
@@ -49,8 +41,7 @@ std::string GLGizmoMove3D::get_tooltip() const
 
 bool GLGizmoMove3D::on_init()
 {
-    for (int i = 0; i < 3; ++i)
-    {
+    for (int i = 0; i < 3; ++i) {
         m_grabbers.push_back(Grabber());
     }
 
@@ -61,7 +52,7 @@ bool GLGizmoMove3D::on_init()
 
 std::string GLGizmoMove3D::on_get_name() const
 {
-    return (_(L("Move")) + " [M]").ToUTF8().data();
+    return _u8L("Move");
 }
 
 bool GLGizmoMove3D::on_is_activable() const
@@ -71,8 +62,7 @@ bool GLGizmoMove3D::on_is_activable() const
 
 void GLGizmoMove3D::on_start_dragging()
 {
-    if (m_hover_id != -1)
-    {
+    if (m_hover_id != -1) {
         m_displacement = Vec3d::Zero();
         const BoundingBoxf3& box = m_parent.get_selection().get_bounding_box();
         m_starting_drag_position = m_grabbers[m_hover_id].center;
@@ -90,14 +80,14 @@ void GLGizmoMove3D::on_stop_dragging()
 void GLGizmoMove3D::on_update(const UpdateData& data)
 {
     if (m_hover_id == 0)
-        m_displacement(0) = calc_projection(data);
+        m_displacement.x() = calc_projection(data);
     else if (m_hover_id == 1)
-        m_displacement(1) = calc_projection(data);
+        m_displacement.y() = calc_projection(data);
     else if (m_hover_id == 2)
-        m_displacement(2) = calc_projection(data);
+        m_displacement.z() = calc_projection(data);
 }
 
-void GLGizmoMove3D::on_render() const
+void GLGizmoMove3D::on_render()
 {
     const Selection& selection = m_parent.get_selection();
 
@@ -108,27 +98,24 @@ void GLGizmoMove3D::on_render() const
     const Vec3d& center = box.center();
 
     // x axis
-    m_grabbers[0].center = Vec3d(box.max(0) + Offset, center(1), center(2));
-    ::memcpy((void*)m_grabbers[0].color, (const void*)&AXES_COLOR[0], 4 * sizeof(float));
+    m_grabbers[0].center = { box.max.x() + Offset, center.y(), center.z() };
+    m_grabbers[0].color = AXES_COLOR[0];
 
     // y axis
-    m_grabbers[1].center = Vec3d(center(0), box.max(1) + Offset, center(2));
-    ::memcpy((void*)m_grabbers[1].color, (const void*)&AXES_COLOR[1], 4 * sizeof(float));
+    m_grabbers[1].center = { center.x(), box.max.y() + Offset, center.z() };
+    m_grabbers[1].color = AXES_COLOR[1];
 
     // z axis
-    m_grabbers[2].center = Vec3d(center(0), center(1), box.max(2) + Offset);
-    ::memcpy((void*)m_grabbers[2].color, (const void*)&AXES_COLOR[2], 4 * sizeof(float));
+    m_grabbers[2].center = { center.x(), center.y(), box.max.z() + Offset };
+    m_grabbers[2].color = AXES_COLOR[2];
 
     glsafe(::glLineWidth((m_hover_id != -1) ? 2.0f : 1.5f));
 
-    if (m_hover_id == -1)
-    {
+    if (m_hover_id == -1) {
         // draw axes
-        for (unsigned int i = 0; i < 3; ++i)
-        {
-            if (m_grabbers[i].enabled)
-            {
-                glsafe(::glColor4fv(AXES_COLOR[i]));
+        for (unsigned int i = 0; i < 3; ++i) {
+            if (m_grabbers[i].enabled) {
+                glsafe(::glColor4fv(AXES_COLOR[i].data()));
                 ::glBegin(GL_LINES);
                 ::glVertex3dv(center.data());
                 ::glVertex3dv(m_grabbers[i].center.data());
@@ -138,29 +125,33 @@ void GLGizmoMove3D::on_render() const
 
         // draw grabbers
         render_grabbers(box);
-        for (unsigned int i = 0; i < 3; ++i)
-        {
+        for (unsigned int i = 0; i < 3; ++i) {
             if (m_grabbers[i].enabled)
                 render_grabber_extension((Axis)i, box, false);
         }
     }
-    else
-    {
+    else {
         // draw axis
-        glsafe(::glColor4fv(AXES_COLOR[m_hover_id]));
+        glsafe(::glColor4fv(AXES_COLOR[m_hover_id].data()));
         ::glBegin(GL_LINES);
         ::glVertex3dv(center.data());
         ::glVertex3dv(m_grabbers[m_hover_id].center.data());
         glsafe(::glEnd());
 
-        // draw grabber
-        float mean_size = (float)((box.size()(0) + box.size()(1) + box.size()(2)) / 3.0);
-        m_grabbers[m_hover_id].render(true, mean_size);
+        GLShaderProgram* shader = wxGetApp().get_shader("gouraud_light");
+        if (shader != nullptr) {
+            shader->start_using();
+            shader->set_uniform("emission_factor", 0.1f);
+            // draw grabber
+            float mean_size = (float)((box.size().x() + box.size().y() + box.size().z()) / 3.0);
+            m_grabbers[m_hover_id].render(true, mean_size);
+            shader->stop_using();
+        }
         render_grabber_extension((Axis)m_hover_id, box, false);
     }
 }
 
-void GLGizmoMove3D::on_render_for_picking() const
+void GLGizmoMove3D::on_render_for_picking()
 {
     glsafe(::glDisable(GL_DEPTH_TEST));
 
@@ -177,8 +168,7 @@ double GLGizmoMove3D::calc_projection(const UpdateData& data) const
 
     Vec3d starting_vec = m_starting_drag_position - m_starting_box_center;
     double len_starting_vec = starting_vec.norm();
-    if (len_starting_vec != 0.0)
-    {
+    if (len_starting_vec != 0.0) {
         Vec3d mouse_dir = data.mouse_ray.unit_vector();
         // finds the intersection of the mouse ray with the plane parallel to the camera viewport and passing throught the starting position
         // use ray-plane intersection see i.e. https://en.wikipedia.org/wiki/Line%E2%80%93plane_intersection algebric form
@@ -200,42 +190,41 @@ double GLGizmoMove3D::calc_projection(const UpdateData& data) const
 
 void GLGizmoMove3D::render_grabber_extension(Axis axis, const BoundingBoxf3& box, bool picking) const
 {
-    if (m_quadric == nullptr)
-        return;
-
-    float mean_size = (float)((box.size()(0) + box.size()(1) + box.size()(2)) / 3.0);
+    float mean_size = (float)((box.size().x() + box.size().y() + box.size().z()) / 3.0);
     double size = m_dragging ? (double)m_grabbers[axis].get_dragging_half_size(mean_size) : (double)m_grabbers[axis].get_half_size(mean_size);
 
-    float color[4];
-    ::memcpy((void*)color, (const void*)m_grabbers[axis].color, 4 * sizeof(float));
-    if (!picking && (m_hover_id != -1))
-    {
+    std::array<float, 4> color = m_grabbers[axis].color;
+    if (!picking && m_hover_id != -1) {
         color[0] = 1.0f - color[0];
         color[1] = 1.0f - color[1];
         color[2] = 1.0f - color[2];
         color[3] = color[3];
     }
 
-    if (!picking)
-        glsafe(::glEnable(GL_LIGHTING));
+    GLShaderProgram* shader = wxGetApp().get_shader("gouraud_light");
+    if (shader == nullptr)
+        return;
+
+    const_cast<GLModel*>(&m_vbo_cone)->set_color(-1, color);
+    if (!picking) {
+        shader->start_using();
+        shader->set_uniform("emission_factor", 0.1f);
+    }
 
-    glsafe(::glColor4fv(color));
     glsafe(::glPushMatrix());
-    glsafe(::glTranslated(m_grabbers[axis].center(0), m_grabbers[axis].center(1), m_grabbers[axis].center(2)));
+    glsafe(::glTranslated(m_grabbers[axis].center.x(), m_grabbers[axis].center.y(), m_grabbers[axis].center.z()));
     if (axis == X)
         glsafe(::glRotated(90.0, 0.0, 1.0, 0.0));
     else if (axis == Y)
         glsafe(::glRotated(-90.0, 1.0, 0.0, 0.0));
 
     glsafe(::glTranslated(0.0, 0.0, 2.0 * size));
-    ::gluQuadricOrientation(m_quadric, GLU_OUTSIDE);
-    ::gluCylinder(m_quadric, 0.75 * size, 0.0, 3.0 * size, 36, 1);
-    ::gluQuadricOrientation(m_quadric, GLU_INSIDE);
-    ::gluDisk(m_quadric, 0.0, 0.75 * size, 36, 1);
+    glsafe(::glScaled(0.75 * size, 0.75 * size, 3.0 * size));
+    m_vbo_cone.render();
     glsafe(::glPopMatrix());
 
-    if (!picking)
-        glsafe(::glDisable(GL_LIGHTING));
+    if (! picking)
+        shader->stop_using();
 }
 
 
