@@ -27,8 +27,6 @@ private:
 
     const float RenderPointScale = 1.f;
 
-    GLUquadricObj* m_quadric;
-
     class CacheEntry {
     public:
         CacheEntry() :
@@ -58,7 +56,7 @@ private:
 
 public:
     GLGizmoSlaSupports(GLCanvas3D& parent, const std::string& icon_filename, unsigned int sprite_id);
-    ~GLGizmoSlaSupports() override;
+    virtual ~GLGizmoSlaSupports() = default;
     void set_sla_support_data(ModelObject* model_object, const Selection& selection);
     bool gizmo_event(SLAGizmoEventType action, const Vec2d& mouse_position, bool shift_down, bool alt_down, bool control_down);
     void delete_selected_points(bool force = false);
@@ -69,11 +67,15 @@ public:
     bool has_backend_supports() const;
     void reslice_SLA_supports(bool postpone_error_messages = false) const;
 
+    bool wants_enter_leave_snapshots() const override { return true; }
+    std::string get_gizmo_entering_text() const override { return _u8L("Entering SLA support points"); }
+    std::string get_gizmo_leaving_text() const override { return _u8L("Leaving SLA support points"); }
+
 private:
     bool on_init() override;
     void on_update(const UpdateData& data) override;
-    void on_render() const override;
-    void on_render_for_picking() const override;
+    void on_render() override;
+    void on_render_for_picking() override;
 
     void render_points(const Selection& selection, bool picking = false) const;
     bool unsaved_changes() const;
@@ -119,6 +121,7 @@ private:
     void auto_generate();
     void switch_to_editing_mode();
     void disable_editing_mode();
+    void ask_about_changes_call_after(std::function<void()> on_yes, std::function<void()> on_no);
 
 protected:
     void on_set_state() override;
