@@ -3,6 +3,7 @@
 
 #include <wx/dataview.h>
 #include <vector>
+#include <map>
 
 #include "ExtraRenderers.hpp"
 
@@ -27,6 +28,7 @@ enum ItemType {
     itSettings      = 16,
     itLayerRoot     = 32,
     itLayer         = 64,
+    itInfo          = 128
 };
 
 enum ColumnNumber
@@ -44,6 +46,16 @@ enum PrintIndicator
     piUnprintable      ,    // unprintable
 };
 
+enum class InfoItemType
+{
+    Undef,
+    CustomSupports,
+    CustomSeam,
+    MmuSegmentation,
+    Sinking,
+    VariableLayerHeight
+};
+
 class ObjectDataViewModelNode;
 WX_DEFINE_ARRAY_PTR(ObjectDataViewModelNode*, MyObjectTreeModelNodePtrArray);
 
@@ -66,9 +78,11 @@ class ObjectDataViewModelNode
     wxBitmap				        m_action_icon;
     PrintIndicator                  m_printable {piUndef};
     wxBitmap				        m_printable_icon;
+    std::string                     m_warning_icon_name{ "" };
 
     std::string                     m_action_icon_name = "";
     ModelVolumeType                 m_volume_type;
+    InfoItemType                    m_info_item_type {InfoItemType::Undef};
 
 public:
     ObjectDataViewModelNode(const wxString& name,
@@ -84,19 +98,11 @@ public:
 
     ObjectDataViewModelNode(ObjectDataViewModelNode* parent,
                             const wxString& sub_obj_name,
+                            Slic3r::ModelVolumeType type,
                             const wxBitmap& bmp,
                             const wxString& extruder,
-                            const int idx = -1 ) :
-        m_parent	(parent),
-        m_name		(sub_obj_name),
-        m_type		(itVolume),
-        m_idx       (idx),
-        m_extruder  (extruder)
-    {
-        m_bmp = bmp;
-        set_action_and_extruder_icons();
-        init_container();
-    }
+                            const int idx = -1,
+                            const std::string& warning_icon_name = std::string());
 
     ObjectDataViewModelNode(ObjectDataViewModelNode* parent,
                             const t_layer_height_range& layer_range,
@@ -104,6 +110,7 @@ public:
                             const wxString& extruder = wxEmptyString );
 
     ObjectDataViewModelNode(ObjectDataViewModelNode* parent, const ItemType type);
+    ObjectDataViewModelNode(ObjectDataViewModelNode* parent, const InfoItemType type);
 
     ~ObjectDataViewModelNode()
     {
@@ -171,15 +178,21 @@ public:
     }
 
     bool            SetValue(const wxVariant &variant, unsigned int col);
-
+    void            SetVolumeType(ModelVolumeType type) { m_volume_type = type; }
     void            SetBitmap(const wxBitmap &icon) { m_bmp = icon; }
+    void            SetExtruder(const wxString &extruder) { m_extruder = extruder; }
+    void            SetWarningBitmap(const wxBitmap& icon, const std::string& warning_icon_name) { m_bmp = icon; m_warning_icon_name = warning_icon_name; }
     const wxBitmap& GetBitmap() const               { return m_bmp; }
     const wxString& GetName() const                 { return m_name; }
     ItemType        GetType() const                 { return m_type; }
+    InfoItemType    GetInfoItemType() const         { return m_info_item_type; }
 	void			SetIdx(const int& idx);
 	int             GetIdx() const                  { return m_idx; }
+    ModelVolumeType GetVolumeType()                 { return m_volume_type; }
 	t_layer_height_range    GetLayerRange() const   { return m_layer_range; }
+    wxString        GetExtruder()                   { return m_extruder; }
     PrintIndicator  IsPrintable() const             { return m_printable; }
+    void            UpdateExtruderAndColorIcon(wxString extruder = "");
 
     // use this function only for childrens
     void AssignAllVal(ObjectDataViewModelNode& from_node)
@@ -215,6 +228,8 @@ public:
     void        set_extruder_icon();
 	// Set printable icon for node
     void        set_printable_icon(PrintIndicator printable);
+    // Set warning icon for node
+    void        set_warning_icon(const std::string& warning_icon);
 
     void        update_settings_digest_bitmaps();
     bool        update_settings_digest(const std::vector<std::string>& categories);
@@ -225,6 +240,7 @@ public:
     bool 		valid();
 #endif /* NDEBUG */
     bool        invalid() const { return m_idx < -1; }
+    bool        has_warning_icon() const { return !m_warning_icon_name.empty(); }
 
 private:
     friend class ObjectDataViewModel;
@@ -241,8 +257,11 @@ wxDECLARE_EVENT(wxCUSTOMEVT_LAST_VOLUME_IS_DELETED, wxCommandEvent);
 class ObjectDataViewModel :public wxDataViewModel
 {
     std::vector<ObjectDataViewModelNode*>       m_objects;
-    std::vector<wxBitmap*>                      m_volume_bmps;
-    wxBitmap*                                   m_warning_bmp { nullptr };
+    std::vector<wxBitmap>                       m_volume_bmps;
+    std::map<InfoItemType, wxBitmap>            m_info_bmps;
+    wxBitmap                                    m_empty_bmp;
+    wxBitmap                                    m_warning_bmp;
+    wxBitmap                                    m_warning_manifold_bmp;
 
     wxDataViewCtrl*                             m_ctrl { nullptr };
 
@@ -252,14 +271,15 @@ public:
 
     wxDataViewItem Add( const wxString &name,
                         const int extruder,
-                        const bool has_errors = false);
+                        const std::string& warning_icon_name = std::string());
     wxDataViewItem AddVolumeChild(  const wxDataViewItem &parent_item,
                                     const wxString &name,
                                     const Slic3r::ModelVolumeType volume_type,
-                                    const bool has_errors = false,
+                                    const std::string& warning_icon_name = std::string(),
                                     const int extruder = 0,
                                     const bool create_frst_child = true);
     wxDataViewItem AddSettingsChild(const wxDataViewItem &parent_item);
+    wxDataViewItem AddInfoChild(const wxDataViewItem &parent_item, InfoItemType info_type);
     wxDataViewItem AddInstanceChild(const wxDataViewItem &parent_item, size_t num);
     wxDataViewItem AddInstanceChild(const wxDataViewItem &parent_item, const std::vector<bool>& print_indicator);
     wxDataViewItem AddLayersRoot(const wxDataViewItem &parent_item);
@@ -267,6 +287,7 @@ public:
                                     const t_layer_height_range& layer_range,
                                     const int extruder = 0,
                                     const int index = -1);
+    size_t         GetItemIndexForFirstVolume(ObjectDataViewModelNode* node_parent);
     wxDataViewItem Delete(const wxDataViewItem &item);
     wxDataViewItem DeleteLastInstance(const wxDataViewItem &parent_item, size_t num);
     void DeleteAll();
@@ -301,7 +322,7 @@ public:
     // helper methods to change the model
 
     unsigned int    GetColumnCount() const override { return 3;}
-    wxString        GetColumnType(unsigned int col) const override{ return wxT("string"); }
+    wxString        GetColumnType(unsigned int col) const override;
 
     void GetValue(  wxVariant &variant,
                     const wxDataViewItem &item,
@@ -314,6 +335,7 @@ public:
                     unsigned int col);
 
     void SetExtruder(const wxString& extruder, wxDataViewItem item);
+    bool SetName    (const wxString& new_name, wxDataViewItem item);
 
     // For parent move child from cur_volume_id place to new_volume_id
     // Remaining items will moved up/down accordingly
@@ -333,13 +355,17 @@ public:
     // Is the container just a header or an item with all columns
     // In our case it is an item with all columns
     bool    HasContainerColumns(const wxDataViewItem& WXUNUSED(item)) const override {	return true; }
+    bool    HasInfoItem(InfoItemType type) const;
 
-    ItemType        GetItemType(const wxDataViewItem &item) const ;
+    ItemType        GetItemType(const wxDataViewItem &item) const;
+    InfoItemType    GetInfoItemType(const wxDataViewItem &item) const;
     wxDataViewItem  GetItemByType(  const wxDataViewItem &parent_item,
                                     ItemType type) const;
     wxDataViewItem  GetSettingsItem(const wxDataViewItem &item) const;
     wxDataViewItem  GetInstanceRootItem(const wxDataViewItem &item) const;
     wxDataViewItem  GetLayerRootItem(const wxDataViewItem &item) const;
+    wxDataViewItem  GetInfoItemByType(const wxDataViewItem &parent_item, InfoItemType type) const;
+
     bool    IsSettingsItem(const wxDataViewItem &item) const;
     void    UpdateSettingsDigest(   const wxDataViewItem &item,
                                     const std::vector<std::string>& categories);
@@ -348,9 +374,8 @@ public:
     void    UpdateObjectPrintable(wxDataViewItem parent_item);
     void    UpdateInstancesPrintable(wxDataViewItem parent_item);
 
-    void    SetVolumeBitmaps(const std::vector<wxBitmap*>& volume_bmps) { m_volume_bmps = volume_bmps; }
-    void    SetWarningBitmap(wxBitmap* bitmap)                          { m_warning_bmp = bitmap; }
     void    SetVolumeType(const wxDataViewItem &item, const Slic3r::ModelVolumeType type);
+    ModelVolumeType GetVolumeType(const wxDataViewItem &item);
     wxDataViewItem SetPrintableState( PrintIndicator printable, int obj_idx,
                                       int subobj_idx = -1, 
                                       ItemType subobj_type = itInstance);
@@ -361,17 +386,24 @@ public:
     void    Rescale();
 
     wxBitmap    GetVolumeIcon(const Slic3r::ModelVolumeType vol_type,
-                              const bool is_marked = false);
+                              const std::string& warning_icon_name = std::string());
+    void        AddWarningIcon(const wxDataViewItem& item, const std::string& warning_name);
     void        DeleteWarningIcon(const wxDataViewItem& item, const bool unmark_object = false);
+    void        UpdateWarningIcon(const wxDataViewItem& item, const std::string& warning_name);
+    bool        HasWarningIcon(const wxDataViewItem& item) const;
     t_layer_height_range    GetLayerRangeByItem(const wxDataViewItem& item) const;
 
     bool        UpdateColumValues(unsigned col);
     void        UpdateExtruderBitmap(wxDataViewItem item);
+    void        UpdateVolumesExtruderBitmap(wxDataViewItem object_item);
+    int         GetDefaultExtruderIdx(wxDataViewItem item);
 
 private:
     wxDataViewItem  AddRoot(const wxDataViewItem& parent_item, const ItemType root_type);
     wxDataViewItem  AddInstanceRoot(const wxDataViewItem& parent_item);
     void            AddAllChildren(const wxDataViewItem& parent);
+
+    wxBitmap&       GetWarningBitmap(const std::string& warning_icon_name);
 };
 
 
