@@ -64,7 +64,7 @@ enum class ActionButtonType : int;
 
 class Sidebar : public wxPanel
 {
-    ConfigOptionMode    m_mode;
+    ConfigOptionMode    m_mode{ConfigOptionMode::comSimple};
 public:
     Sidebar(Plater *parent);
     Sidebar(Sidebar &&) = delete;
@@ -111,7 +111,7 @@ public:
     void                    update_mode();
     bool                    is_collapsed();
     void                    collapse(bool collapse);
-    void                    update_searcher();
+    void                    check_and_update_searcher(bool respect_mode = false);
     void                    update_ui_from_settings();
 
 #ifdef _MSW_DARK_MODE
@@ -323,9 +323,7 @@ public:
     bool can_replace_with_stl() const;
     bool can_mirror() const;
     bool can_split(bool to_objects) const;
-#if ENABLE_ENHANCED_PRINT_VOLUME_FIT
     bool can_scale_to_print_volume() const;
-#endif // ENABLE_ENHANCED_PRINT_VOLUME_FIT
 
     void msw_rescale();
     void sys_color_changed();
