@@ -13,6 +13,13 @@
 #include <wx/colordlg.h>
 #include <wx/wupdlock.h>
 #include <wx/menu.h>
+#include <wx/odcombo.h>
+#include <wx/listbook.h>
+
+#ifdef _WIN32
+#include <wx/msw/dcclient.h>
+#include <wx/msw/private.h>
+#endif
 
 #include "libslic3r/libslic3r.h"
 #include "libslic3r/PrintConfig.hpp"
@@ -31,6 +38,7 @@
 #include "BitmapCache.hpp"
 #include "PhysicalPrinterDialog.hpp"
 #include "SavePresetDialog.hpp"
+#include "MsgDialog.hpp"
 
 // A workaround for a set of issues related to text fitting into gtk widgets:
 // See e.g.: https://github.com/prusa3d/PrusaSlicer/issues/4584
@@ -62,20 +70,13 @@ namespace GUI {
  * control size calculation methods (virtual) are overridden.
  **/
 
-PresetComboBox::PresetComboBox(wxWindow* parent, Preset::Type preset_type, const wxSize& size) :
-    wxBitmapComboBox(parent, wxID_ANY, wxEmptyString, wxDefaultPosition, size, 0, nullptr, wxCB_READONLY),
+PresetComboBox::PresetComboBox(wxWindow* parent, Preset::Type preset_type, const wxSize& size, PresetBundle* preset_bundle/* = nullptr*/) :
+    BitmapComboBox(parent, wxID_ANY, wxEmptyString, wxDefaultPosition, size, 0, nullptr, wxCB_READONLY),
     m_type(preset_type),
     m_last_selected(wxNOT_FOUND),
     m_em_unit(em_unit(this)),
-    m_preset_bundle(wxGetApp().preset_bundle)
+    m_preset_bundle(preset_bundle ? preset_bundle : wxGetApp().preset_bundle)
 {
-    SetFont(wxGetApp().normal_font());
-#ifdef _WIN32
-    // Workaround for ignoring CBN_EDITCHANGE events, which are processed after the content of the combo box changes, so that
-    // the index of the item inside CBN_EDITCHANGE may no more be valid.
-    EnableTextChangedEvents(false);
-#endif /* _WIN32 */
-
     switch (m_type)
     {
     case Preset::TYPE_PRINT: {
@@ -121,23 +122,26 @@ PresetComboBox::PresetComboBox(wxWindow* parent, Preset::Type preset_type, const
     Bind(wxEVT_COMBOBOX_DROPDOWN, [this](wxCommandEvent&) { m_suppress_change = false; });
     Bind(wxEVT_COMBOBOX_CLOSEUP,  [this](wxCommandEvent&) { m_suppress_change = true;  });
 
-    Bind(wxEVT_COMBOBOX, [this](wxCommandEvent& evt) {
-        // see https://github.com/prusa3d/PrusaSlicer/issues/3889
-        // Under OSX: in case of use of a same names written in different case (like "ENDER" and "Ender")
-        // m_presets_choice->GetSelection() will return first item, because search in PopupListCtrl is case-insensitive.
-        // So, use GetSelection() from event parameter 
-        auto selected_item = evt.GetSelection();
+    Bind(wxEVT_COMBOBOX, &PresetComboBox::OnSelect, this);
+}
 
-        auto marker = reinterpret_cast<Marker>(this->GetClientData(selected_item));
-        if (marker >= LABEL_ITEM_DISABLED && marker < LABEL_ITEM_MAX)
-            this->SetSelection(this->m_last_selected);
-        else if (on_selection_changed && (m_last_selected != selected_item || m_collection->current_is_dirty())) {
-            m_last_selected = selected_item;
-            on_selection_changed(selected_item);
-            evt.StopPropagation();
-        }
-        evt.Skip();
-    });
+void PresetComboBox::OnSelect(wxCommandEvent& evt)
+{
+    // see https://github.com/prusa3d/PrusaSlicer/issues/3889
+    // Under OSX: in case of use of a same names written in different case (like "ENDER" and "Ender")
+    // m_presets_choice->GetSelection() will return first item, because search in PopupListCtrl is case-insensitive.
+    // So, use GetSelection() from event parameter 
+    auto selected_item = evt.GetSelection();
+
+    auto marker = reinterpret_cast<Marker>(this->GetClientData(selected_item));
+    if (marker >= LABEL_ITEM_DISABLED && marker < LABEL_ITEM_MAX)
+        this->SetSelection(m_last_selected);
+    else if (on_selection_changed && (m_last_selected != selected_item || m_collection->current_is_dirty())) {
+        m_last_selected = selected_item;
+        on_selection_changed(selected_item);
+        evt.StopPropagation();
+    }
+    evt.Skip();
 }
 
 PresetComboBox::~PresetComboBox()
@@ -186,6 +190,12 @@ void PresetComboBox::update_selection()
     validate_selection();
 
     SetSelection(m_last_selected);
+#ifdef __WXMSW__
+    // From the Windows 2004 the tooltip for preset combobox doesn't work after next call of SetTooltip()
+    // (There was an issue, when tooltip doesn't appears after changing of the preset selection)
+    // But this workaround seems to work: We should to kill tooltip and than set new tooltip value
+    SetToolTip(NULL);
+#endif
     SetToolTip(GetString(m_last_selected));
 
 // A workaround for a set of issues related to text fitting into gtk widgets:
@@ -201,13 +211,28 @@ void PresetComboBox::update_selection()
 
     if (!cell) return;
 
-    g_object_set(G_OBJECT(cell), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+    g_object_set(G_OBJECT(cell), "ellipsize", PANGO_ELLIPSIZE_END, (char*)NULL);
 
     // Only the list of cells must be freed, the renderer isn't ours to free
     g_list_free(cells);
 #endif
 }
 
+static std::string suffix(const Preset& preset)
+{
+    return (preset.is_dirty ? Preset::suffix_modified() : "");
+}
+
+static std::string suffix(Preset* preset)
+{
+    return (preset->is_dirty ? Preset::suffix_modified() : "");
+}
+
+wxString PresetComboBox::get_preset_name(const Preset & preset)
+{
+    return from_u8(preset.name/* + suffix(preset)*/);
+}
+
 void PresetComboBox::update(std::string select_preset_name)
 {
     Freeze();
@@ -226,7 +251,7 @@ void PresetComboBox::update(std::string select_preset_name)
     for (size_t i = presets.front().is_visible ? 0 : m_collection->num_default_presets(); i < presets.size(); ++i)
     {
         const Preset& preset = presets[i];
-        if (!preset.is_visible || !preset.is_compatible)
+        if (!m_show_all && (!preset.is_visible || !preset.is_compatible))
             continue;
 
         // marker used for disable incompatible printer models for the selected physical printer
@@ -246,17 +271,17 @@ void PresetComboBox::update(std::string select_preset_name)
         assert(bmp);
 
         if (!is_enabled)
-            incomp_presets.emplace(wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), bmp);
+            incomp_presets.emplace(get_preset_name(preset), bmp);
         else if (preset.is_default || preset.is_system)
         {
-            Append(wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), *bmp);
+            Append(get_preset_name(preset), *bmp);
             validate_selection(preset.name == select_preset_name);
         }
         else
         {
-            nonsys_presets.emplace(wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), std::pair<wxBitmap*, bool>(bmp, is_enabled));
+            nonsys_presets.emplace(get_preset_name(preset), std::pair<wxBitmap*, bool>(bmp, is_enabled));
             if (preset.name == select_preset_name || (select_preset_name.empty() && is_enabled))
-                selected = wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str());
+                selected = get_preset_name(preset);
         }
         if (i + 1 == m_collection->num_default_presets())
             set_label_marker(Append(separator(L("System presets")), wxNullBitmap));
@@ -311,7 +336,8 @@ bool PresetComboBox::del_physical_printer(const wxString& note_string/* = wxEmpt
         msg += note_string + "\n";
     msg += format_wxstr(_L("Are you sure you want to delete \"%1%\" printer?"), printer_name);
 
-    if (wxMessageDialog(this, msg, _L("Delete Physical Printer"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION).ShowModal() != wxID_YES)
+    //if (wxMessageDialog(this, msg, _L("Delete Physical Printer"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION).ShowModal() != wxID_YES)
+    if (MessageDialog(this, msg, _L("Delete Physical Printer"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION).ShowModal() != wxID_YES)
         return false;
 
     m_preset_bundle->physical_printers.delete_selected_printer();
@@ -329,11 +355,22 @@ bool PresetComboBox::del_physical_printer(const wxString& note_string/* = wxEmpt
     return true;
 }
 
+void PresetComboBox::show_all(bool show_all)
+{
+    m_show_all = show_all;
+    update();
+}
+
 void PresetComboBox::update()
 {
     this->update(into_u8(this->GetString(this->GetSelection())));
 }
 
+void PresetComboBox::update_from_bundle()
+{
+    this->update(m_collection->get_selected_preset().name);
+}
+
 void PresetComboBox::msw_rescale()
 {
     m_em_unit = em_unit(this);
@@ -348,6 +385,12 @@ void PresetComboBox::msw_rescale()
     update();
 }
 
+void PresetComboBox::sys_color_changed()
+{
+    wxGetApp().UpdateDarkUI(this);
+    msw_rescale();
+}
+
 void PresetComboBox::fill_width_height()
 {
     // To avoid asserts, each added bitmap to wxBitmapCombobox should be the same size, so
@@ -364,9 +407,9 @@ void PresetComboBox::fill_width_height()
     thin_icon_width = lroundf(8 * scale_f);          // analogue to 8px;
     wide_icon_width = norm_icon_width + thin_icon_width;
 
-    space_icon_width = lroundf(2 * scale_f);
-    thin_space_icon_width = 2 * space_icon_width;
-    wide_space_icon_width = 3 * space_icon_width;
+    space_icon_width      = lroundf(2 * scale_f);
+    thin_space_icon_width = lroundf(4 * scale_f);
+    wide_space_icon_width = lroundf(6 * scale_f);
 }
 
 wxString PresetComboBox::separator(const std::string& label)
@@ -376,7 +419,7 @@ wxString PresetComboBox::separator(const std::string& label)
 
 wxBitmap* PresetComboBox::get_bmp(  std::string bitmap_key, bool wide_icons, const std::string& main_icon_name, 
                                     bool is_compatible/* = true*/, bool is_system/* = false*/, bool is_single_bar/* = false*/,
-                                    std::string filament_rgb/* = ""*/, std::string extruder_rgb/* = ""*/)
+                                    const std::string& filament_rgb/* = ""*/, const std::string& extruder_rgb/* = ""*/, const std::string& material_rgb/* = ""*/)
 {
     // If the filament preset is not compatible and there is a "red flag" icon loaded, show it left
     // to the filament color image.
@@ -385,8 +428,10 @@ wxBitmap* PresetComboBox::get_bmp(  std::string bitmap_key, bool wide_icons, con
 
     bitmap_key += is_system ? ",syst" : ",nsyst";
     bitmap_key += ",h" + std::to_string(icon_height);
-    if (wxGetApp().dark_mode())
+    bool dark_mode = wxGetApp().dark_mode();
+    if (dark_mode)
         bitmap_key += ",dark";
+    bitmap_key += material_rgb;
 
     wxBitmap* bmp = bitmap_cache().find(bitmap_key);
     if (bmp == nullptr) {
@@ -401,10 +446,10 @@ wxBitmap* PresetComboBox::get_bmp(  std::string bitmap_key, bool wide_icons, con
             unsigned char rgb[3];
             // Paint the color bars.
             bitmap_cache().parse_color(filament_rgb, rgb);
-            bmps.emplace_back(bitmap_cache().mksolid(is_single_bar ? wide_icon_width : norm_icon_width, icon_height, rgb));
+            bmps.emplace_back(bitmap_cache().mksolid(is_single_bar ? wide_icon_width : norm_icon_width, icon_height, rgb, false, 1, dark_mode));
             if (!is_single_bar) {
                 bitmap_cache().parse_color(extruder_rgb, rgb);
-                bmps.emplace_back(bitmap_cache().mksolid(thin_icon_width, icon_height, rgb));
+                bmps.emplace_back(bitmap_cache().mksolid(thin_icon_width, icon_height, rgb, false, 1, dark_mode));
             }
             // Paint a lock at the system presets.
             bmps.emplace_back(bitmap_cache().mkclear(space_icon_width, icon_height));
@@ -413,7 +458,10 @@ wxBitmap* PresetComboBox::get_bmp(  std::string bitmap_key, bool wide_icons, con
         {
             // Paint the color bars.
             bmps.emplace_back(bitmap_cache().mkclear(thin_space_icon_width, icon_height));
-            bmps.emplace_back(create_scaled_bitmap(main_icon_name));
+            if (m_type == Preset::TYPE_SLA_MATERIAL)
+                bmps.emplace_back(create_scaled_bitmap(main_icon_name, this, 16, false, material_rgb));
+            else
+                bmps.emplace_back(create_scaled_bitmap(main_icon_name));
             // Paint a lock at the system presets.
             bmps.emplace_back(bitmap_cache().mkclear(wide_space_icon_width, icon_height));
         }
@@ -496,72 +544,6 @@ bool PresetComboBox::selection_is_changed_according_to_physical_printers()
     return true;
 }
 
-#ifdef __APPLE__
-bool PresetComboBox::OnAddBitmap(const wxBitmap& bitmap)
-{
-    if (bitmap.IsOk())
-    {
-        // we should use scaled! size values of bitmap
-        int width = (int)bitmap.GetScaledWidth();
-        int height = (int)bitmap.GetScaledHeight();
-
-        if (m_usedImgSize.x < 0)
-        {
-            // If size not yet determined, get it from this image.
-            m_usedImgSize.x = width;
-            m_usedImgSize.y = height;
-
-            // Adjust control size to vertically fit the bitmap
-            wxWindow* ctrl = GetControl();
-            ctrl->InvalidateBestSize();
-            wxSize newSz = ctrl->GetBestSize();
-            wxSize sz = ctrl->GetSize();
-            if (newSz.y > sz.y)
-                ctrl->SetSize(sz.x, newSz.y);
-            else
-                DetermineIndent();
-        }
-
-        wxCHECK_MSG(width == m_usedImgSize.x && height == m_usedImgSize.y,
-            false,
-            "you can only add images of same size");
-
-        return true;
-    }
-
-    return false;
-}
-
-void PresetComboBox::OnDrawItem(wxDC& dc,
-    const wxRect& rect,
-    int item,
-    int flags) const
-{
-    const wxBitmap& bmp = *(static_cast<wxBitmap*>(m_bitmaps[item]));
-    if (bmp.IsOk())
-    {
-        // we should use scaled! size values of bitmap
-        wxCoord w = bmp.GetScaledWidth();
-        wxCoord h = bmp.GetScaledHeight();
-
-        const int imgSpacingLeft = 4;
-
-        // Draw the image centered
-        dc.DrawBitmap(bmp,
-            rect.x + (m_usedImgSize.x - w) / 2 + imgSpacingLeft,
-            rect.y + (rect.height - h) / 2,
-            true);
-    }
-
-    wxString text = GetString(item);
-    if (!text.empty())
-        dc.DrawText(text,
-            rect.x + m_imgAreaWidth + 1,
-            rect.y + (rect.height - dc.GetCharHeight()) / 2);
-}
-#endif
-
-
 // ---------------------------------
 // ***  PlaterPresetComboBox  ***
 // ---------------------------------
@@ -569,34 +551,6 @@ void PresetComboBox::OnDrawItem(wxDC& dc,
 PlaterPresetComboBox::PlaterPresetComboBox(wxWindow *parent, Preset::Type preset_type) :
     PresetComboBox(parent, preset_type, wxSize(15 * wxGetApp().em_unit(), -1))
 {
-    Bind(wxEVT_COMBOBOX, [this](wxCommandEvent &evt) {
-        auto selected_item = evt.GetSelection();
-
-        auto marker = reinterpret_cast<Marker>(this->GetClientData(selected_item));
-        if (marker >= LABEL_ITEM_MARKER && marker < LABEL_ITEM_MAX) {
-            this->SetSelection(this->m_last_selected);
-            evt.StopPropagation();
-            if (marker == LABEL_ITEM_WIZARD_PRINTERS)
-                show_add_menu();
-            else
-            {
-                ConfigWizard::StartPage sp = ConfigWizard::SP_WELCOME;
-                switch (marker) {
-                case LABEL_ITEM_WIZARD_FILAMENTS: sp = ConfigWizard::SP_FILAMENTS; break;
-                case LABEL_ITEM_WIZARD_MATERIALS: sp = ConfigWizard::SP_MATERIALS; break;
-                default: break;
-                }
-                wxTheApp->CallAfter([sp]() { wxGetApp().run_wizard(ConfigWizard::RR_USER, sp); });
-            }
-        } else if (marker == LABEL_ITEM_PHYSICAL_PRINTER || this->m_last_selected != selected_item || m_collection->current_is_dirty() ) {
-            this->m_last_selected = selected_item;
-            evt.SetInt(this->m_type);
-            evt.Skip();
-        } else {
-            evt.StopPropagation();
-        }
-    });
-
     if (m_type == Preset::TYPE_FILAMENT)
     {
         Bind(wxEVT_LEFT_DOWN, [this](wxMouseEvent &event) {
@@ -619,31 +573,7 @@ PlaterPresetComboBox::PlaterPresetComboBox(wxWindow *parent, Preset::Type preset
             }
 
             // Swallow the mouse click and open the color picker.
-
-            // get current color
-            DynamicPrintConfig* cfg = wxGetApp().get_tab(Preset::TYPE_PRINTER)->get_config();
-            auto colors = static_cast<ConfigOptionStrings*>(cfg->option("extruder_colour")->clone());
-            wxColour clr(colors->values[m_extruder_idx]);
-            if (!clr.IsOk())
-                clr = wxColour(0,0,0); // Don't set alfa to transparence
-
-            auto data = new wxColourData();
-            data->SetChooseFull(1);
-            data->SetColour(clr);
-
-            wxColourDialog dialog(this, data);
-            dialog.CenterOnParent();
-            if (dialog.ShowModal() == wxID_OK)
-            {
-                colors->values[m_extruder_idx] = dialog.GetColourData().GetColour().GetAsString(wxC2S_HTML_SYNTAX).ToStdString();
-
-                DynamicPrintConfig cfg_new = *cfg;
-                cfg_new.set_key_value("extruder_colour", colors);
-
-                wxGetApp().get_tab(Preset::TYPE_PRINTER)->load_config(cfg_new);
-                this->update();
-                wxGetApp().plater()->on_config_change(cfg_new);
-            }
+            change_extruder_color();
         });
     }
 
@@ -653,28 +583,15 @@ PlaterPresetComboBox::PlaterPresetComboBox(wxWindow *parent, Preset::Type preset
     edit_btn->Bind(wxEVT_BUTTON, [this](wxCommandEvent)
     {
         // In a case of a physical printer, for its editing open PhysicalPrinterDialog
-        if (m_type == Preset::TYPE_PRINTER/* && this->is_selected_physical_printer()*/) {
-            this->show_edit_menu();
-            return;
-        }
-
-        if (!switch_to_tab())
-            return;
-
-        /* In a case of a multi-material printing, for editing another Filament Preset
-         * it's needed to select this preset for the "Filament settings" Tab
-         */
-        if (m_type == Preset::TYPE_FILAMENT && wxGetApp().extruders_edited_cnt() > 1)
-        {
-            const std::string& selected_preset = GetString(GetSelection()).ToUTF8().data();
-
-            // Call select_preset() only if there is new preset and not just modified
-            if ( !boost::algorithm::ends_with(selected_preset, Preset::suffix_modified()) )
-            {
-                const std::string& preset_name = wxGetApp().preset_bundle->filaments.get_preset_name_by_alias(selected_preset);
-                wxGetApp().get_tab(m_type)->select_preset(preset_name);
-            }
-        }
+        if (m_type == Preset::TYPE_PRINTER
+#ifdef __linux__
+            // To edit extruder color from the sidebar
+            || m_type == Preset::TYPE_FILAMENT
+#endif //__linux__
+            )
+            show_edit_menu();
+        else
+            switch_to_tab();
     });
 }
 
@@ -684,20 +601,93 @@ PlaterPresetComboBox::~PlaterPresetComboBox()
         edit_btn->Destroy();
 }
 
-bool PlaterPresetComboBox::switch_to_tab()
+static void run_wizard(ConfigWizard::StartPage sp)
+{
+    wxGetApp().run_wizard(ConfigWizard::RR_USER, sp);
+}
+
+void PlaterPresetComboBox::OnSelect(wxCommandEvent &evt)
+{
+    auto selected_item = evt.GetSelection();
+
+    auto marker = reinterpret_cast<Marker>(this->GetClientData(selected_item));
+    if (marker >= LABEL_ITEM_MARKER && marker < LABEL_ITEM_MAX) {
+        this->SetSelection(m_last_selected);
+        evt.StopPropagation();
+        if (marker == LABEL_ITEM_MARKER)
+            return;
+        if (marker == LABEL_ITEM_WIZARD_PRINTERS)
+            show_add_menu();
+        else {
+            ConfigWizard::StartPage sp = ConfigWizard::SP_WELCOME;
+            switch (marker) {
+            case LABEL_ITEM_WIZARD_FILAMENTS: sp = ConfigWizard::SP_FILAMENTS; break;
+            case LABEL_ITEM_WIZARD_MATERIALS: sp = ConfigWizard::SP_MATERIALS; break;
+            default: break;
+            }
+            wxTheApp->CallAfter([sp]() { run_wizard(sp); });
+        }
+        return;
+    }
+    else if (marker == LABEL_ITEM_PHYSICAL_PRINTER || m_last_selected != selected_item || m_collection->current_is_dirty())
+        m_last_selected = selected_item;
+        
+    evt.Skip();
+}
+
+void PlaterPresetComboBox::switch_to_tab()
 {
     Tab* tab = wxGetApp().get_tab(m_type);
     if (!tab)
-        return false;
+        return;
 
-    int page_id = wxGetApp().tab_panel()->FindPage(tab);
-    if (page_id == wxNOT_FOUND)
-        return false;
+    if (int page_id = wxGetApp().tab_panel()->FindPage(tab); page_id != wxNOT_FOUND)
+    {
+        wxGetApp().tab_panel()->SetSelection(page_id);
+        // Switch to Settings NotePad
+        wxGetApp().mainframe->select_tab();
 
-    wxGetApp().tab_panel()->SetSelection(page_id);
-    // Switch to Settings NotePad
-    wxGetApp().mainframe->select_tab();
-    return true;
+        //In a case of a multi-material printing, for editing another Filament Preset
+        //it's needed to select this preset for the "Filament settings" Tab
+        if (m_type == Preset::TYPE_FILAMENT && wxGetApp().extruders_edited_cnt() > 1)
+        {
+            const std::string& selected_preset = GetString(GetSelection()).ToUTF8().data();
+            // Call select_preset() only if there is new preset and not just modified
+            if (!boost::algorithm::ends_with(selected_preset, Preset::suffix_modified()))
+            {
+                const std::string& preset_name = wxGetApp().preset_bundle->filaments.get_preset_name_by_alias(selected_preset);
+                wxGetApp().get_tab(m_type)->select_preset(preset_name);
+            }
+        }
+    }
+}
+
+void PlaterPresetComboBox::change_extruder_color()
+{
+    // get current color
+    DynamicPrintConfig* cfg = wxGetApp().get_tab(Preset::TYPE_PRINTER)->get_config();
+    auto colors = static_cast<ConfigOptionStrings*>(cfg->option("extruder_colour")->clone());
+    wxColour clr(colors->values[m_extruder_idx]);
+    if (!clr.IsOk())
+        clr = wxColour(0, 0, 0); // Don't set alfa to transparence
+
+    auto data = new wxColourData();
+    data->SetChooseFull(1);
+    data->SetColour(clr);
+
+    wxColourDialog dialog(this, data);
+    dialog.CenterOnParent();
+    if (dialog.ShowModal() == wxID_OK)
+    {
+        colors->values[m_extruder_idx] = dialog.GetColourData().GetColour().GetAsString(wxC2S_HTML_SYNTAX).ToStdString();
+
+        DynamicPrintConfig cfg_new = *cfg;
+        cfg_new.set_key_value("extruder_colour", colors);
+
+        wxGetApp().get_tab(Preset::TYPE_PRINTER)->load_config(cfg_new);
+        this->update();
+        wxGetApp().plater()->on_config_change(cfg_new);
+    }
 }
 
 void PlaterPresetComboBox::show_add_menu()
@@ -705,8 +695,8 @@ void PlaterPresetComboBox::show_add_menu()
     wxMenu* menu = new wxMenu();
 
     append_menu_item(menu, wxID_ANY, _L("Add/Remove presets"), "",
-        [this](wxCommandEvent&) { 
-            wxTheApp->CallAfter([]() { wxGetApp().run_wizard(ConfigWizard::RR_USER, ConfigWizard::SP_PRINTERS); });
+        [](wxCommandEvent&) {
+            wxTheApp->CallAfter([]() { run_wizard(ConfigWizard::SP_PRINTERS); });
         }, "edit_uni", menu, []() { return true; }, wxGetApp().plater());
 
     append_menu_item(menu, wxID_ANY, _L("Add physical printer"), "",
@@ -726,6 +716,16 @@ void PlaterPresetComboBox::show_edit_menu()
     append_menu_item(menu, wxID_ANY, _L("Edit preset"), "",
         [this](wxCommandEvent&) { this->switch_to_tab(); }, "cog", menu, []() { return true; }, wxGetApp().plater());
 
+#ifdef __linux__
+    // To edit extruder color from the sidebar
+    if (m_type == Preset::TYPE_FILAMENT) {
+        append_menu_item(menu, wxID_ANY, _L("Change extruder color"), "",
+            [this](wxCommandEvent&) { this->change_extruder_color(); }, "funnel", menu, []() { return true; }, wxGetApp().plater());
+        wxGetApp().plater()->PopupMenu(menu);
+        return;
+    }
+#endif //__linux__
+
     if (this->is_selected_physical_printer()) {
         append_menu_item(menu, wxID_ANY, _L("Edit physical printer"), "",
             [this](wxCommandEvent&) { this->edit_physical_printer(); }, "cog", menu, []() { return true; }, wxGetApp().plater());
@@ -735,8 +735,8 @@ void PlaterPresetComboBox::show_edit_menu()
     }
     else
         append_menu_item(menu, wxID_ANY, _L("Add/Remove presets"), "",
-            [this](wxCommandEvent&) {
-                wxTheApp->CallAfter([]() { wxGetApp().run_wizard(ConfigWizard::RR_USER, ConfigWizard::SP_PRINTERS); });
+            [](wxCommandEvent&) {
+                wxTheApp->CallAfter([]() { run_wizard(ConfigWizard::SP_PRINTERS); });
             }, "edit_uni", menu, []() { return true; }, wxGetApp().plater());
 
     append_menu_item(menu, wxID_ANY, _L("Add physical printer"), "",
@@ -745,6 +745,12 @@ void PlaterPresetComboBox::show_edit_menu()
     wxGetApp().plater()->PopupMenu(menu);
 }
 
+wxString PlaterPresetComboBox::get_preset_name(const Preset& preset)
+{
+    std::string name = preset.alias.empty() ? preset.name : preset.alias;
+    return from_u8(name + suffix(preset));
+}
+
 // Only the compatible presets are shown.
 // If an incompatible preset is selected, it is shown as well.
 void PlaterPresetComboBox::update()
@@ -780,8 +786,8 @@ void PlaterPresetComboBox::update()
 
     std::map<wxString, wxBitmap*> nonsys_presets;
 
-    wxString selected_user_preset = "";
-    wxString tooltip = "";
+    wxString selected_user_preset;
+    wxString tooltip;
     const std::deque<Preset>& presets = m_collection->get_presets();
 
     if (!presets.front().is_visible)
@@ -799,7 +805,7 @@ void PlaterPresetComboBox::update()
         if (!preset.is_visible || (!preset.is_compatible && !is_selected))
             continue;
 
-        std::string bitmap_key, filament_rgb, extruder_rgb;
+        std::string bitmap_key, filament_rgb, extruder_rgb, material_rgb;
         std::string bitmap_type_name = bitmap_key = m_type == Preset::TYPE_PRINTER && preset.printer_technology() == ptSLA ? "sla_printer" : m_main_bitmap_name;
 
         bool single_bar = false;
@@ -813,25 +819,30 @@ void PlaterPresetComboBox::update()
 
             bitmap_key += single_bar ? filament_rgb : filament_rgb + extruder_rgb;
         }
+        else if (m_type == Preset::TYPE_SLA_MATERIAL) {
+            material_rgb = is_selected ? m_preset_bundle->sla_materials.get_edited_preset().config.opt_string("material_colour") : preset.config.opt_string("material_colour");
+            if (material_rgb.empty())
+                material_rgb = print_config_def.get("material_colour")->get_default_value<ConfigOptionString>()->value;
+        }
 
         wxBitmap* bmp = get_bmp(bitmap_key, wide_icons, bitmap_type_name, 
                                 preset.is_compatible, preset.is_system || preset.is_default, 
-                                single_bar, filament_rgb, extruder_rgb);
+                                single_bar, filament_rgb, extruder_rgb, material_rgb);
         assert(bmp);
 
         const std::string name = preset.alias.empty() ? preset.name : preset.alias;
         if (preset.is_default || preset.is_system) {
-            Append(wxString::FromUTF8((name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), *bmp);
+            Append(get_preset_name(preset), *bmp);
             validate_selection(is_selected);
             if (is_selected)
-                tooltip = wxString::FromUTF8(preset.name.c_str());
+                tooltip = from_u8(preset.name);
         }
         else
         {
-            nonsys_presets.emplace(wxString::FromUTF8((name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), bmp);
+            nonsys_presets.emplace(get_preset_name(preset), bmp);
             if (is_selected) {
-                selected_user_preset = wxString::FromUTF8((name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str());
-                tooltip = wxString::FromUTF8(preset.name.c_str());
+                selected_user_preset = get_preset_name(preset);
+                tooltip = from_u8(preset.name);
             }
         }
         if (i + 1 == m_collection->num_default_presets())
@@ -854,15 +865,15 @@ void PlaterPresetComboBox::update()
             const PhysicalPrinterCollection& ph_printers = m_preset_bundle->physical_printers;
 
             for (PhysicalPrinterCollection::ConstIterator it = ph_printers.begin(); it != ph_printers.end(); ++it) {
-                for (const std::string preset_name : it->get_preset_names()) {
+                for (const std::string& preset_name : it->get_preset_names()) {
                     Preset* preset = m_collection->find_preset(preset_name);
-                    if (!preset)
+                    if (!preset || !preset->is_visible)
                         continue;
                     std::string main_icon_name, bitmap_key = main_icon_name = preset->printer_technology() == ptSLA ? "sla_printer" : m_main_bitmap_name;
                     wxBitmap* bmp = get_bmp(main_icon_name, wide_icons, main_icon_name);
                     assert(bmp);
 
-                    set_label_marker(Append(wxString::FromUTF8((it->get_full_name(preset_name) + (preset->is_dirty ? Preset::suffix_modified() : "")).c_str()), *bmp), LABEL_ITEM_PHYSICAL_PRINTER);
+                    set_label_marker(Append(from_u8(it->get_full_name(preset_name) + suffix(preset)), *bmp), LABEL_ITEM_PHYSICAL_PRINTER);
                     validate_selection(ph_printers.is_selected(it, preset_name));
                 }
             }
@@ -884,12 +895,24 @@ void PlaterPresetComboBox::update()
     update_selection();
     Thaw();
 
-    if (!tooltip.IsEmpty())
+    if (!tooltip.IsEmpty()) {
+#ifdef __WXMSW__
+        // From the Windows 2004 the tooltip for preset combobox doesn't work after next call of SetTooltip()
+        // (There was an issue, when tooltip doesn't appears after changing of the preset selection)
+        // But this workaround seems to work: We should to kill tooltip and than set new tooltip value
+        // See, https://groups.google.com/g/wx-users/c/mOEe3fgHrzk
+        SetToolTip(NULL);
+#endif
         SetToolTip(tooltip);
+    }
 
+#ifdef __WXMSW__
+    // Use this part of code just on Windows to avoid of some layout issues on Linux
+    // see https://github.com/prusa3d/PrusaSlicer/issues/5163 and https://github.com/prusa3d/PrusaSlicer/issues/5505
     // Update control min size after rescale (changed Display DPI under MSW)
     if (GetMinWidth() != 20 * m_em_unit)
         SetMinSize(wxSize(20 * m_em_unit, GetSize().GetHeight()));
+#endif //__WXMSW__
 }
 
 void PlaterPresetComboBox::msw_rescale()
@@ -906,40 +929,47 @@ void PlaterPresetComboBox::msw_rescale()
 TabPresetComboBox::TabPresetComboBox(wxWindow* parent, Preset::Type preset_type) :
     PresetComboBox(parent, preset_type, wxSize(35 * wxGetApp().em_unit(), -1))
 {
-    Bind(wxEVT_COMBOBOX, [this](wxCommandEvent& evt) {
-        // see https://github.com/prusa3d/PrusaSlicer/issues/3889
-        // Under OSX: in case of use of a same names written in different case (like "ENDER" and "Ender")
-        // m_presets_choice->GetSelection() will return first item, because search in PopupListCtrl is case-insensitive.
-        // So, use GetSelection() from event parameter 
-        auto selected_item = evt.GetSelection();
+}
 
-        auto marker = reinterpret_cast<Marker>(this->GetClientData(selected_item));
-        if (marker >= LABEL_ITEM_DISABLED && marker < LABEL_ITEM_MAX) {
-            this->SetSelection(this->m_last_selected);
-            if (marker == LABEL_ITEM_WIZARD_PRINTERS)
-                wxTheApp->CallAfter([this]() {
-                wxGetApp().run_wizard(ConfigWizard::RR_USER, ConfigWizard::SP_PRINTERS);
+void TabPresetComboBox::OnSelect(wxCommandEvent &evt)
+{
+    // see https://github.com/prusa3d/PrusaSlicer/issues/3889
+    // Under OSX: in case of use of a same names written in different case (like "ENDER" and "Ender")
+    // m_presets_choice->GetSelection() will return first item, because search in PopupListCtrl is case-insensitive.
+    // So, use GetSelection() from event parameter 
+    auto selected_item = evt.GetSelection();
 
-                // update combobox if its parent is a PhysicalPrinterDialog
-                PhysicalPrinterDialog* parent = dynamic_cast<PhysicalPrinterDialog*>(this->GetParent());
-                if (parent != nullptr)
-                    update();
-            });
-        }
-        else if (on_selection_changed && (m_last_selected != selected_item || m_collection->current_is_dirty()) ) {
-            m_last_selected = selected_item;
-            on_selection_changed(selected_item);
-        }
+    auto marker = reinterpret_cast<Marker>(this->GetClientData(selected_item));
+    if (marker >= LABEL_ITEM_DISABLED && marker < LABEL_ITEM_MAX) {
+        this->SetSelection(m_last_selected);
+        if (marker == LABEL_ITEM_WIZARD_PRINTERS)
+            wxTheApp->CallAfter([this]() {
+            run_wizard(ConfigWizard::SP_PRINTERS);
 
-        evt.StopPropagation();
+            // update combobox if its parent is a PhysicalPrinterDialog
+            PhysicalPrinterDialog* parent = dynamic_cast<PhysicalPrinterDialog*>(this->GetParent());
+            if (parent != nullptr)
+                update();
+        });
+    }
+    else if (on_selection_changed && (m_last_selected != selected_item || m_collection->current_is_dirty())) {
+        m_last_selected = selected_item;
+        on_selection_changed(selected_item);
+    }
+
+    evt.StopPropagation();
 #ifdef __WXMSW__
-        // From the Win 2004 preset combobox lose a focus after change the preset selection
-        // and that is why the up/down arrow doesn't work properly
-        // (see https://github.com/prusa3d/PrusaSlicer/issues/5531 ).
-        // So, set the focus to the combobox explicitly
-        this->SetFocus();
-#endif    
-    });
+    // From the Win 2004 preset combobox lose a focus after change the preset selection
+    // and that is why the up/down arrow doesn't work properly
+    // (see https://github.com/prusa3d/PrusaSlicer/issues/5531 ).
+    // So, set the focus to the combobox explicitly
+    this->SetFocus();
+#endif
+}
+
+wxString TabPresetComboBox::get_preset_name(const Preset& preset)
+{
+    return from_u8(preset.name + suffix(preset));
 }
 
 // Update the choice UI from the list of presets.
@@ -987,7 +1017,7 @@ void TabPresetComboBox::update()
         assert(bmp);
 
         if (preset.is_default || preset.is_system) {
-            int item_id = Append(wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), *bmp);
+            int item_id = Append(get_preset_name(preset), *bmp);
             if (!is_enabled)
                 set_label_marker(item_id, LABEL_ITEM_DISABLED);
             validate_selection(i == idx_selected);
@@ -995,9 +1025,9 @@ void TabPresetComboBox::update()
         else
         {
             std::pair<wxBitmap*, bool> pair(bmp, is_enabled);
-            nonsys_presets.emplace(wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str()), std::pair<wxBitmap*, bool>(bmp, is_enabled));
+            nonsys_presets.emplace(get_preset_name(preset), std::pair<wxBitmap*, bool>(bmp, is_enabled));
             if (i == idx_selected)
-                selected = wxString::FromUTF8((preset.name + (preset.is_dirty ? Preset::suffix_modified() : "")).c_str());
+                selected = get_preset_name(preset);
         }
         if (i + 1 == m_collection->num_default_presets())
             set_label_marker(Append(separator(L("System presets")), wxNullBitmap));
@@ -1022,16 +1052,16 @@ void TabPresetComboBox::update()
             const PhysicalPrinterCollection& ph_printers = m_preset_bundle->physical_printers;
 
             for (PhysicalPrinterCollection::ConstIterator it = ph_printers.begin(); it != ph_printers.end(); ++it) {
-                for (const std::string preset_name : it->get_preset_names()) {
+                for (const std::string& preset_name : it->get_preset_names()) {
                     Preset* preset = m_collection->find_preset(preset_name);
-                    if (!preset)
+                    if (!preset || !preset->is_visible)
                         continue;
                     std::string main_icon_name = preset->printer_technology() == ptSLA ? "sla_printer" : m_main_bitmap_name;
 
                     wxBitmap* bmp = get_bmp(main_icon_name, main_icon_name, "", true, true, false);
                     assert(bmp);
 
-                    set_label_marker(Append(wxString::FromUTF8((it->get_full_name(preset_name) + (preset->is_dirty ? Preset::suffix_modified() : "")).c_str()), *bmp), LABEL_ITEM_PHYSICAL_PRINTER);
+                    set_label_marker(Append(from_u8(it->get_full_name(preset_name) + suffix(preset)), *bmp), LABEL_ITEM_PHYSICAL_PRINTER);
                     validate_selection(ph_printers.is_selected(it, preset_name));
                 }
             }
@@ -1078,15 +1108,15 @@ void TabPresetComboBox::update_dirty()
             preset_name = PhysicalPrinter::get_preset_name(preset_name);
         }
             
-        const Preset* preset = m_collection->find_preset(preset_name, false);
+        Preset* preset = m_collection->find_preset(preset_name, false);
         if (preset) {
-            std::string new_label = preset->is_dirty ? preset->name + Preset::suffix_modified() : preset->name;
+            std::string new_label = preset->name + suffix(preset);
 
             if (marker == LABEL_ITEM_PHYSICAL_PRINTER)
                 new_label = ph_printer_name + PhysicalPrinter::separator() + new_label;
 
             if (old_label != new_label)
-                SetString(ui_id, wxString::FromUTF8(new_label.c_str()));
+                SetString(ui_id, from_u8(new_label));
         }
     }
 #ifdef __APPLE__
