@@ -3,6 +3,8 @@
 
 #include <string>
 
+class wxProgressDialog;
+
 namespace Slic3r {
 
 class Model;
@@ -12,12 +14,14 @@ class Print;
 #ifdef HAS_WIN10SDK
 
 extern bool is_windows10();
-extern void fix_model_by_win10_sdk_gui(ModelObject &model_object, int volume_idx);
+// returt false, if fixing was canceled
+extern bool fix_model_by_win10_sdk_gui(ModelObject &model_object, int volume_idx, wxProgressDialog& progress_dlg, const wxString& msg_header, std::string& fix_result);
 
 #else /* HAS_WIN10SDK */
 
 inline bool is_windows10() { return false; }
-inline void fix_model_by_win10_sdk_gui(ModelObject &, int) {}
+// returt false, if fixing was canceled
+inline bool fix_model_by_win10_sdk_gui(ModelObject&, int, wxProgressDialog&, const wxString&, std::string&) { return false; }
 
 #endif /* HAS_WIN10SDK */
 
