@@ -20,7 +20,7 @@ public:
     OctoPrint(DynamicPrintConfig *config);
     ~OctoPrint() override = default;
 
-    const char* get_name() const;
+    const char* get_name() const override;
 
     bool test(wxString &curl_msg) const override;
     wxString get_test_ok_msg () const override;
@@ -28,18 +28,19 @@ public:
     bool upload(PrintHostUpload upload_data, ProgressFn prorgess_fn, ErrorFn error_fn) const override;
     bool has_auto_discovery() const override { return true; }
     bool can_test() const override { return true; }
-    bool can_start_print() const override { return true; }
-    std::string get_host() const override { return host; }
-    const std::string& get_apikey() const { return apikey; }
-    const std::string& get_cafile() const { return cafile; }
+    PrintHostPostUploadActions get_post_upload_actions() const override { return PrintHostPostUploadAction::StartPrint; }
+    std::string get_host() const override { return m_host; }
+    const std::string& get_apikey() const { return m_apikey; }
+    const std::string& get_cafile() const { return m_cafile; }
 
 protected:
     virtual bool validate_version_text(const boost::optional<std::string> &version_text) const;
 
 private:
-    std::string host;
-    std::string apikey;
-    std::string cafile;
+    std::string m_host;
+    std::string m_apikey;
+    std::string m_cafile;
+    bool        m_ssl_revoke_best_effort;
 
     virtual void set_auth(Http &http) const;
     std::string make_url(const std::string &path) const;
@@ -55,7 +56,7 @@ public:
 
     wxString get_test_ok_msg() const override;
     wxString get_test_failed_msg(wxString &msg) const override;
-    bool can_start_print() const override { return false; }
+    PrintHostPostUploadActions get_post_upload_actions() const override { return {}; }
 
 protected:
     bool validate_version_text(const boost::optional<std::string> &version_text) const override;
@@ -64,10 +65,35 @@ private:
     void set_auth(Http &http) const override;
 
     // Host authorization type.
-    AuthorizationType authorization_type;
+    AuthorizationType m_authorization_type;
     // username and password for HTTP Digest Authentization (RFC RFC2617)
-    std::string username;
-    std::string password;
+    std::string m_username;
+    std::string m_password;
+};
+
+class PrusaLink : public OctoPrint
+{
+public:
+    PrusaLink(DynamicPrintConfig* config);
+    ~PrusaLink() override = default;
+
+    const char* get_name() const override;
+
+    wxString get_test_ok_msg() const override;
+    wxString get_test_failed_msg(wxString& msg) const override;
+    PrintHostPostUploadActions get_post_upload_actions() const override { return PrintHostPostUploadAction::StartPrint; }
+
+protected:
+    bool validate_version_text(const boost::optional<std::string>& version_text) const override;
+
+private:
+    void set_auth(Http& http) const override;
+
+    // Host authorization type.
+    AuthorizationType m_authorization_type;
+    // username and password for HTTP Digest Authentization (RFC RFC2617)
+    std::string m_username;
+    std::string m_password;
 };
 
 }
