@@ -2,12 +2,14 @@
 #define slic3r_PrintHost_hpp_
 
 #include <memory>
+#include <set>
 #include <string>
 #include <functional>
 #include <boost/filesystem/path.hpp>
 
 #include <wx/string.h>
 
+#include <libslic3r/enum_bitmask.hpp>
 #include "Http.hpp"
 
 class wxArrayString;
@@ -16,6 +18,13 @@ namespace Slic3r {
 
 class DynamicPrintConfig;
 
+enum class PrintHostPostUploadAction {
+    None,
+    StartPrint,
+    StartSimulation
+};
+using PrintHostPostUploadActions = enum_bitmask<PrintHostPostUploadAction>;
+ENABLE_ENUM_BITMASK_OPERATORS(PrintHostPostUploadAction);
 
 struct PrintHostUpload
 {
@@ -24,10 +33,9 @@ struct PrintHostUpload
     
     std::string group;
     
-    bool start_print = false;
+    PrintHostPostUploadAction post_action { PrintHostPostUploadAction::None };
 };
 
-
 class PrintHost
 {
 public:
@@ -44,7 +52,7 @@ public:
     virtual bool upload(PrintHostUpload upload_data, ProgressFn prorgess_fn, ErrorFn error_fn) const = 0;
     virtual bool has_auto_discovery() const = 0;
     virtual bool can_test() const = 0;
-    virtual bool can_start_print() const = 0;
+    virtual PrintHostPostUploadActions get_post_upload_actions() const = 0;
     // A print host usually does not support multiple printers, with the exception of Repetier server.
     virtual bool supports_multiple_printers() const { return false; }
     virtual std::string get_host() const = 0;
